@@ -13,7 +13,7 @@
     Artifex Software, Inc.,  101 Lucas Valley Road #110,
     San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 
-    $Id: jbig2_text.c 8022 2007-06-05 22:23:38Z giles $
+    $Id: jbig2_text.c 8320 2007-10-25 22:14:22Z giles $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -112,7 +112,7 @@ jbig2_decode_text_region(Jbig2Ctx *ctx, Jbig2Segment *segment,
     Jbig2Image *IB;
     Jbig2WordStream *ws = NULL;
     Jbig2HuffmanState *hs = NULL;
-    Jbig2HuffmanTable *SBSYMCODES;
+    Jbig2HuffmanTable *SBSYMCODES = NULL;
     Jbig2ArithState *as = NULL;
     Jbig2ArithIntCtx *IADT = NULL;
     Jbig2ArithIntCtx *IAFS = NULL;
