@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2007 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: dxmain.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: dxmain.c 8377 2007-11-18 09:30:55Z ghostgum $ */
 
 /* dxmain.c */
 /* 
@@ -691,7 +691,6 @@ static int display_sync(void *handle, void *device)
 	(color == DISPLAY_COLORS_SEPARATION)) {
 	/* check if separations have changed */
 	int i;
-	int num_visible = 0;
 	gchar *str;
 	for (i=0; i<IMAGE_DEVICEN_MAX; i++) {
 	    gtk_label_get(
@@ -705,13 +704,7 @@ static int display_sync(void *handle, void *device)
 		    img->devicen[i].name);
 		gtk_widget_show(img->separation[i]);
 	    }
-	    if (img->devicen[i].used && img->devicen[i].visible)
-		num_visible++;
 	}
-	if (num_visible <= 1)
-	    gtk_widget_show(img->separation[i]);
-	else
-	    gtk_widget_hide(img->separation[i]);
     }
 		
     /* some formats need to be converted for use by GdkRgb */
