@@ -1,22 +1,16 @@
-/* Copyright (C) 1989, 1992, 1993, 1994, 1995, 1996, 1997, 1998 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdebug.h,v 1.3.2.1.2.1 2003/01/17 00:49:00 giles Exp $ */
+/* $Id: gdebug.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Debugging machinery definitions */
 
 #ifndef gdebug_INCLUDED
@@ -54,7 +48,7 @@
 
 /* Define the array of debugging flags, indexed by character code. */
 extern char gs_debug[128];
-bool gs_debug_c(P1(int /*char */ ));
+bool gs_debug_c(int /*char */ );
 
 /*
  * Define an alias for a specialized debugging flag
@@ -117,11 +111,11 @@ extern FILE *gs_debug_out;
 #endif
 
 /* Debugging support procedures in gsmisc.c */
-void debug_dump_bytes(P3(const byte * from, const byte * to,
-			 const char *msg));
-void debug_dump_bitmap(P4(const byte * from, uint raster, uint height,
-			  const char *msg));
-void debug_print_string(P2(const byte * str, uint len));
-void debug_print_string_hex(P2(const byte * str, uint len));
+void debug_dump_bytes(const byte * from, const byte * to,
+		      const char *msg);
+void debug_dump_bitmap(const byte * from, uint raster, uint height,
+		       const char *msg);
+void debug_print_string(const byte * str, uint len);
+void debug_print_string_hex(const byte * str, uint len);
 
 #endif /* gdebug_INCLUDED */
