@@ -1,27 +1,22 @@
-/* Copyright (C) 1995, 1996, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevcgm.c,v 1.2.6.2.2.1 2003/01/17 00:49:00 giles Exp $ */
+/* $Id: gdevcgm.c 8250 2007-09-25 13:31:24Z giles $ */
 /* CGM (Computer Graphics Metafile) driver */
 #include "memory_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gxdevice.h"
+#include "gp.h"
 #include "gsparam.h"
 #include "gdevcgml.h"
 #include "gdevpccm.h"
@@ -32,11 +27,9 @@
 	  do masked copy_mono with cell array if possible
  ****************/
 
-#define fname_size 80
-
 typedef struct gx_device_cgm_s {
     gx_device_common;
-    char fname[fname_size + 1];
+    char fname[gp_file_name_sizeof];
     FILE *file;
     cgm_state *st;
     bool in_picture;
@@ -48,21 +41,21 @@ gs_private_st_suffix_add1_final(st_device_cgm, gx_device_cgm,
 				gx_device_finalize, st_device, st);
 
 /* Device procedures */
-private dev_proc_open_device(cgm_open);
-private dev_proc_output_page(cgm_output_page);
-private dev_proc_close_device(cgm_close);
-private dev_proc_fill_rectangle(cgm_fill_rectangle);
+static dev_proc_open_device(cgm_open);
+static dev_proc_output_page(cgm_output_page);
+static dev_proc_close_device(cgm_close);
+static dev_proc_fill_rectangle(cgm_fill_rectangle);
 
 #if 0
-private dev_proc_tile_rectangle(cgm_tile_rectangle);
+static dev_proc_tile_rectangle(cgm_tile_rectangle);
 
 #else
 #define cgm_tile_rectangle NULL
 #endif
-private dev_proc_copy_mono(cgm_copy_mono);
-private dev_proc_copy_color(cgm_copy_color);
-private dev_proc_get_params(cgm_get_params);
-private dev_proc_put_params(cgm_put_params);
+static dev_proc_copy_mono(cgm_copy_mono);
+static dev_proc_copy_color(cgm_copy_color);
+static dev_proc_get_params(cgm_get_params);
+static dev_proc_put_params(cgm_put_params);
 
 /* In principle, all the drawing operations should be polymorphic, */
 /* but it's just as easy just to test the depth, since we're not */
@@ -97,7 +90,7 @@ cgm_device("cgmmono", 1, 1, 2,
 	   gx_default_map_rgb_color, gx_default_w_b_map_color_rgb);
 
 gx_device_cgm gs_cgm8_device =
-cgm_device("cgm8", 8, 6, 7,
+cgm_device("cgm8", 8, 5, 6,
 	   pc_8bit_map_rgb_color, pc_8bit_map_color_rgb);
 
 gx_device_cgm gs_cgm24_device =
@@ -105,14 +98,14 @@ cgm_device("cgm24", 24, 255, 255,
 	   gx_default_rgb_map_rgb_color, gx_default_rgb_map_color_rgb);
 
 /* Define allocator procedures for the CGM library. */
-private void *
+static void *
 cgm_gs_alloc(void *private_data, uint size)
 {
     gx_device_cgm *cdev = private_data;
 
     return gs_alloc_bytes(cdev->memory, size, "cgm_gs_alloc");
 }
-private void
+static void
 cgm_gs_free(void *private_data, void *obj)
 {
     gx_device_cgm *cdev = private_data;
@@ -123,7 +116,7 @@ cgm_gs_free(void *private_data, void *obj)
 /* ---------------- Utilities ---------------- */
 
 /* Convert a CGM result code to our error values. */
-private int
+static int
 cgm_error_code(cgm_result result)
 {
     switch (result) {
@@ -142,7 +135,7 @@ cgm_error_code(cgm_result result)
 /* ---------------- Device control ---------------- */
 
 /* Open the device */
-private int
+static int
 cgm_open(gx_device * dev)
 {
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
@@ -188,7 +181,7 @@ cgm_open(gx_device * dev)
 }
 
 /* Output a page */
-private int
+static int
 cgm_output_page(gx_device * dev, int num_copies, int flush)
 {
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
@@ -204,7 +197,7 @@ cgm_output_page(gx_device * dev, int num_copies, int flush)
 }
 
 /* Close the device */
-private int
+static int
 cgm_close(gx_device * dev)
 {
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
@@ -224,7 +217,7 @@ cgm_close(gx_device * dev)
 }
 
 /* Get parameters.  CGM devices add OutputFile to the default set. */
-private int
+static int
 cgm_get_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
@@ -240,7 +233,7 @@ cgm_get_params(gx_device * dev, gs_param_list * plist)
 }
 
 /* Put parameters. */
-private int
+static int
 cgm_put_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
@@ -251,11 +244,13 @@ cgm_put_params(gx_device * dev, gs_param_list * plist)
 
     switch (code = param_read_string(plist, (param_name = "OutputFile"), &ofs)) {
 	case 0:
-	    if (dev->LockSafetyParams) {
+	    if (dev->LockSafetyParams &&
+		    bytes_compare(ofs.data, ofs.size,
+			(const byte *)cdev->fname, strlen(cdev->fname))) {
 	        ecode = gs_note_error(gs_error_invalidaccess);
 		goto ofe;
 	    }
-	    if (ofs.size > fname_size)
+	    if (ofs.size >= gp_file_name_sizeof)
 		ecode = gs_error_limitcheck;
 	    else
 		break;
@@ -309,7 +304,7 @@ cgm_put_params(gx_device * dev, gs_param_list * plist)
 /* Begin a picture if necessary. */
 #define begin_picture(cdev)\
   if ( !cdev->in_picture ) cgm_begin_picture(cdev)
-private int
+static int
 cgm_begin_picture(gx_device_cgm * cdev)
 {
     cgm_picture_elements pic;
@@ -368,7 +363,7 @@ cgm_begin_picture(gx_device_cgm * cdev)
 }
 
 /* Convert a gx_color_index to a CGM color. */
-private void
+static void
 cgm_color_from_color_index(cgm_color * pcc, const gx_device_cgm * cdev,
 			   gx_color_index color)
 {
@@ -382,7 +377,7 @@ cgm_color_from_color_index(cgm_color * pcc, const gx_device_cgm * cdev,
 }
 
 /* Fill a rectangle. */
-private int
+static int
 cgm_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 		   gx_color_index color)
 {
@@ -390,13 +385,12 @@ cgm_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     cgm_color fill_color;
     cgm_point points[2];
     cgm_result result;
-
+    
     fit_fill(dev, x, y, w, h);
     if (!cdev->in_picture) {	/* Check for erasepage. */
-	if (color == (*dev_proc(dev, map_rgb_color)) (dev,
-				     gx_max_color_value, gx_max_color_value,
-						      gx_max_color_value)
-	    )
+	gx_color_value blank[3] = {gx_max_color_value, gx_max_color_value, 
+				   gx_max_color_value};
+	if (color == (*dev_proc(dev, encode_color)) (dev, blank))
 	    return 0;
 	cgm_begin_picture(cdev);
     }
@@ -413,7 +407,7 @@ cgm_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 
 #if 0
 /* Tile a rectangle.  We should do this with a pattern if possible. */
-private int
+static int
 cgm_tile_rectangle(gx_device * dev, const gx_tile_bitmap * tile,
 	int x, int y, int w, int h, gx_color_index zero, gx_color_index one,
 		   int px, int py)
@@ -428,7 +422,7 @@ cgm_tile_rectangle(gx_device * dev, const gx_tile_bitmap * tile,
 /* the destination region is still white; if so, we can use a cell array */
 /* (or, even better, a pattern).  However, we still need the slow method */
 /* for the case where we don't know the background color or it isn't white. */
-private int
+static int
 cgm_copy_mono(gx_device * dev,
 	      const byte * base, int sourcex, int raster, gx_bitmap_id id,
 	int x, int y, int w, int h, gx_color_index zero, gx_color_index one)
@@ -475,7 +469,7 @@ cgm_copy_mono(gx_device * dev,
 }
 
 /* Copy a color bitmap. */
-private int
+static int
 cgm_copy_color(gx_device * dev,
 	       const byte * base, int sourcex, int raster, gx_bitmap_id id,
 	       int x, int y, int w, int h)
