@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevm24.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: gdevm24.c 8250 2007-09-25 13:31:24Z giles $ */
 /* 24-bit-per-pixel "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -54,7 +54,7 @@ static uint prev_colors[256];
 
 /* Procedures */
 declare_mem_procs(mem_true24_copy_mono, mem_true24_copy_color, mem_true24_fill_rectangle);
-private dev_proc_copy_alpha(mem_true24_copy_alpha);
+static dev_proc_copy_alpha(mem_true24_copy_alpha);
 
 /* The device descriptor. */
 const gx_device_memory mem_true24_device =
@@ -99,7 +99,7 @@ mem_full_alpha_device("image24", 24, 0, mem_open,
 #endif
 
 /* Fill a rectangle with a color. */
-private int
+static int
 mem_true24_fill_rectangle(gx_device * dev,
 			  int x, int y, int w, int h, gx_color_index color)
 {
@@ -339,7 +339,7 @@ mem_true24_fill_rectangle(gx_device * dev,
 }
 
 /* Copy a monochrome bitmap. */
-private int
+static int
 mem_true24_copy_mono(gx_device * dev,
 	       const byte * base, int sourcex, int sraster, gx_bitmap_id id,
 	int x, int y, int w, int h, gx_color_index zero, gx_color_index one)
@@ -456,7 +456,7 @@ mem_true24_copy_mono(gx_device * dev,
 }
 
 /* Copy a color bitmap. */
-private int
+static int
 mem_true24_copy_color(gx_device * dev,
 	       const byte * base, int sourcex, int sraster, gx_bitmap_id id,
 		      int x, int y, int w, int h)
@@ -469,7 +469,7 @@ mem_true24_copy_color(gx_device * dev,
 }
 
 /* Copy an alpha map. */
-private int
+static int
 mem_true24_copy_alpha(gx_device * dev, const byte * base, int sourcex,
 		   int sraster, gx_bitmap_id id, int x, int y, int w, int h,
 		      gx_color_index color, int depth)
@@ -532,7 +532,7 @@ mem_full_device("image24w", 24, 0, mem_open,
 		gx_no_strip_copy_rop, mem_word_get_bits_rectangle);
 
 /* Fill a rectangle with a color. */
-private int
+static int
 mem24_word_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 			  gx_color_index color)
 {
@@ -550,7 +550,7 @@ mem24_word_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 }
 
 /* Copy a bitmap. */
-private int
+static int
 mem24_word_copy_mono(gx_device * dev,
 	       const byte * base, int sourcex, int sraster, gx_bitmap_id id,
 	int x, int y, int w, int h, gx_color_index zero, gx_color_index one)
@@ -572,7 +572,7 @@ mem24_word_copy_mono(gx_device * dev,
 }
 
 /* Copy a color bitmap. */
-private int
+static int
 mem24_word_copy_color(gx_device * dev,
 	       const byte * base, int sourcex, int sraster, gx_bitmap_id id,
 		      int x, int y, int w, int h)
