@@ -1,22 +1,16 @@
-/* Copyright (C) 1992, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevpbm.c,v 1.3.6.2.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevpbm.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Portable Bit/Gray/PixMap drivers */
 #include "gdevprn.h"
 #include "gscdefs.h"
@@ -108,34 +102,34 @@ typedef struct gx_device_pbm_s gx_device_pbm;
 }
 
 /* For all but PBM, we need our own color mapping and alpha procedures. */
-private dev_proc_map_rgb_color(pgm_map_rgb_color);
-private dev_proc_map_rgb_color(ppm_map_rgb_color);
-private dev_proc_map_color_rgb(pgm_map_color_rgb);
-private dev_proc_map_color_rgb(ppm_map_color_rgb);
-private dev_proc_map_cmyk_color(pkm_map_cmyk_color);
-private dev_proc_map_color_rgb(pkm_map_color_rgb);
-private dev_proc_get_params(ppm_get_params);
-private dev_proc_put_params(ppm_put_params);
-private dev_proc_copy_alpha(pnm_copy_alpha);
-private dev_proc_begin_typed_image(pnm_begin_typed_image);
+static dev_proc_map_rgb_color(pgm_map_rgb_color);
+static dev_proc_map_rgb_color(ppm_map_rgb_color);
+static dev_proc_map_color_rgb(pgm_map_color_rgb);
+static dev_proc_map_color_rgb(ppm_map_color_rgb);
+static dev_proc_map_cmyk_color(pkm_map_cmyk_color);
+static dev_proc_map_color_rgb(pkm_map_color_rgb);
+static dev_proc_get_params(ppm_get_params);
+static dev_proc_put_params(ppm_put_params);
+static dev_proc_copy_alpha(pnm_copy_alpha);
+static dev_proc_begin_typed_image(pnm_begin_typed_image);
 
 /* We need to initialize uses_color when opening the device, */
 /* and after each showpage. */
-private dev_proc_open_device(ppm_open);
-private dev_proc_output_page(ppm_output_page);
+static dev_proc_open_device(ppm_open);
+static dev_proc_output_page(ppm_output_page);
 
 /* And of course we need our own print-page routines. */
-private dev_proc_print_page(pbm_print_page);
-private dev_proc_print_page(pgm_print_page);
-private dev_proc_print_page(ppm_print_page);
-private dev_proc_print_page(pkm_print_page);
-private dev_proc_print_page(psm_print_page);
-private dev_proc_print_page(psm_print_page);
-private dev_proc_print_page(pam_print_page);
+static dev_proc_print_page(pbm_print_page);
+static dev_proc_print_page(pgm_print_page);
+static dev_proc_print_page(ppm_print_page);
+static dev_proc_print_page(pkm_print_page);
+static dev_proc_print_page(psm_print_page);
+static dev_proc_print_page(psm_print_page);
+static dev_proc_print_page(pam_print_page);
 
-private int pam_print_row(gx_device_printer * pdev, byte * data, int depth,
+static int pam_print_row(gx_device_printer * pdev, byte * data, int depth,
 	       FILE * pstream);
-private int pam_print_page(gx_device_printer * pdev, FILE * pstream);
+static int pam_print_page(gx_device_printer * pdev, FILE * pstream);
 
 /* The device procedures */
 
@@ -147,22 +141,22 @@ private int pam_print_page(gx_device_printer * pdev, FILE * pstream);
 	p_map_cmyk_color, NULL, NULL, NULL, gx_page_device_get_page_device\
 }
 
-private const gx_device_procs pbm_procs =
+static const gx_device_procs pbm_procs =
     pgpm_procs(gdev_prn_open, gdev_prn_get_params,
 	       gdev_prn_map_rgb_color, gdev_prn_map_color_rgb, NULL);
-private const gx_device_procs pgm_procs =
+static const gx_device_procs pgm_procs =
     pgpm_procs(ppm_open, gdev_prn_get_params,
 	       pgm_map_rgb_color, pgm_map_color_rgb, NULL);
-private const gx_device_procs ppm_procs =
+static const gx_device_procs ppm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       gx_default_rgb_map_rgb_color, ppm_map_color_rgb, NULL);
-private const gx_device_procs pnm_procs =
+static const gx_device_procs pnm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       ppm_map_rgb_color, ppm_map_color_rgb, NULL);
-private const gx_device_procs pkm_procs =
+static const gx_device_procs pkm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       NULL, cmyk_1bit_map_color_rgb, cmyk_1bit_map_cmyk_color);
-private const gx_device_procs pam_procs =
+static const gx_device_procs pam_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       NULL, cmyk_8bit_map_color_rgb, cmyk_8bit_map_cmyk_color);
 
@@ -221,7 +215,7 @@ pbm_prn_device(pbm_procs, "plan9bm", '9', 1, 1, 1, 1, 1, 1,
 /* ------ Initialization ------ */
 
 /* Set the copy_alpha and color mapping procedures if necessary. */
-private void
+static void
 ppm_set_dev_procs(gx_device * pdev)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -254,7 +248,7 @@ ppm_set_dev_procs(gx_device * pdev)
  * a planar device.
  */
 
-private int
+static int
 ppm_open(gx_device * pdev)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -262,13 +256,15 @@ ppm_open(gx_device * pdev)
 
     if (code < 0)
 	return code;
+    pdev->color_info.separable_and_linear = GX_CINFO_SEP_LIN;
+    set_linear_color_bits_mask_shift(pdev);
     bdev->uses_color = 0;
     ppm_set_dev_procs(pdev);
     return code;
 }
 
 /* Print a page, and reset uses_color if this is a showpage. */
-private int
+static int
 ppm_output_page(gx_device * pdev, int num_copies, int flush)
 {
     int code = gdev_prn_output_page(pdev, num_copies, flush);
@@ -285,16 +281,23 @@ ppm_output_page(gx_device * pdev, int num_copies, int flush)
 
 /* Map an RGB color to a PGM gray value. */
 /* Keep track of whether the image is black-and-white or gray. */
-private gx_color_index
-pgm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
-		  gx_color_value b)
+static gx_color_index
+pgm_map_rgb_color(gx_device * pdev, const gx_color_value cv[])
 {				/* We round the value rather than truncating it. */
-    gx_color_value gray =
-    ((r * (ulong) lum_red_weight) +
+    gx_color_value gray;
+    /* TO_DO_DEVICEN  - Kludge to emulate pre DeviceN math errors */
+#if 1
+    gx_color_value r, g, b;
+
+    r = cv[0]; g = cv[0]; b = cv[0];
+    gray = ((r * (ulong) lum_red_weight) +
      (g * (ulong) lum_green_weight) +
      (b * (ulong) lum_blue_weight) +
      (lum_all_weights / 2)) / lum_all_weights
     * pdev->color_info.max_gray / gx_max_color_value;
+#else	    /* Should be ... */
+    gray = cv[0] * pdev->color_info.max_gray / gx_max_color_value;
+#endif
 
     if (!(gray == 0 || gray == pdev->color_info.max_gray)) {
 	gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -305,7 +308,7 @@ pgm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
 }
 
 /* Map a PGM gray value back to an RGB color. */
-private int
+static int
 pgm_map_color_rgb(gx_device * dev, gx_color_index color,
 		  gx_color_value prgb[3])
 {
@@ -318,18 +321,42 @@ pgm_map_color_rgb(gx_device * dev, gx_color_index color,
     return 0;
 }
 
+/*
+ * Pre gs8.00 version of RGB mapping for 24-bit true (RGB) color devices
+ * It is kept here for backwards comparibility since the gs8.00 version
+ * has changed in functionality.  The new one requires that the device be
+ * 'separable'.  This routine is logically separable but does not require
+ * the various color_info fields associated with separability (comp_shift,
+ * comp_bits, and comp_mask) be setup.
+ */
+
+static gx_color_index
+gx_old_default_rgb_map_rgb_color(gx_device * dev,
+		       gx_color_value r, gx_color_value g, gx_color_value b)
+{
+    if (dev->color_info.depth == 24)
+	return gx_color_value_to_byte(b) +
+	    ((uint) gx_color_value_to_byte(g) << 8) +
+	    ((ulong) gx_color_value_to_byte(r) << 16);
+    else {
+	int bpc = dev->color_info.depth / 3;
+	int drop = sizeof(gx_color_value) * 8 - bpc;
+
+	return ((((r >> drop) << bpc) + (g >> drop)) << bpc) + (b >> drop);
+    }
+}
+
 /* Map an RGB color to a PPM color tuple. */
 /* Keep track of whether the image is black-and-white, gray, or colored. */
-private gx_color_index
-ppm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
-		  gx_color_value b)
+static gx_color_index
+ppm_map_rgb_color(gx_device * pdev, const gx_color_value cv[])
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
-    gx_color_index color = gx_default_rgb_map_rgb_color(pdev, r, g, b);
-    int bpc = pdev->color_info.depth / 3;
+    gx_color_index color = 
+	    gx_old_default_rgb_map_rgb_color(pdev, cv[0], cv[1], cv[2]);
+    uint bpc = pdev->color_info.depth / 3;
     gx_color_index mask =
 	((gx_color_index)1 << (pdev->color_info.depth - bpc)) - 1;
-
     if (!(((color >> bpc) ^ color) & mask)) { /* gray shade */
 	if (color != 0 && (~color & mask))
 	    bdev->uses_color |= 1;
@@ -339,7 +366,7 @@ ppm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
 }
 
 /* Map a PPM color tuple back to an RGB color. */
-private int
+static int
 ppm_map_color_rgb(gx_device * dev, gx_color_index color,
 		  gx_color_value prgb[3])
 {
@@ -357,16 +384,15 @@ ppm_map_color_rgb(gx_device * dev, gx_color_index color,
 }
 
 /* Map a CMYK color to a pixel value. */
-private gx_color_index
-pkm_map_cmyk_color(gx_device * pdev, gx_color_value c, gx_color_value m,
-		   gx_color_value y, gx_color_value k)
+static gx_color_index
+pkm_map_cmyk_color(gx_device * pdev, const gx_color_value cv[])
 {
     uint bpc = pdev->color_info.depth >> 2;
     uint max_value = pdev->color_info.max_color;
-    uint cc = c * max_value / gx_max_color_value;
-    uint mc = m * max_value / gx_max_color_value;
-    uint yc = y * max_value / gx_max_color_value;
-    uint kc = k * max_value / gx_max_color_value;
+    uint cc = cv[0] * max_value / gx_max_color_value;
+    uint mc = cv[1] * max_value / gx_max_color_value;
+    uint yc = cv[2] * max_value / gx_max_color_value;
+    uint kc = cv[3] * max_value / gx_max_color_value;
     gx_color_index color =
 	(((((cc << bpc) + mc) << bpc) + yc) << bpc) + kc;
 
@@ -374,7 +400,7 @@ pkm_map_cmyk_color(gx_device * pdev, gx_color_value c, gx_color_value m,
 }
 
 /* Map a CMYK pixel value to RGB. */
-private int
+static int
 pkm_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value rgb[3])
 {
     int bpc = dev->color_info.depth >> 2;
@@ -399,15 +425,19 @@ pkm_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value rgb[3])
 
 /* Augment get/put_params to add UsePlanarBuffer */
 
-private int
+static int
 ppm_get_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
+    int code;
 
-    return gdev_prn_get_params_planar(pdev, plist, &bdev->UsePlanarBuffer);
+    code = gdev_prn_get_params_planar(pdev, plist, &bdev->UsePlanarBuffer);
+    if (code < 0) return code;
+    code = param_write_null(plist, "OutputIntent");
+    return code;
 }
 
-private int
+static int
 ppm_put_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -417,8 +447,37 @@ ppm_put_params(gx_device * pdev, gs_param_list * plist)
     int ecode = 0;
     int code;
     long v;
+    gs_param_string_array intent;
     const char *vname;
 
+    if ((code = param_read_string_array(plist, "OutputIntent", &intent)) == 0) {
+	/* This device does not use the OutputIntent parameter.
+	   We include this code just as a sample how to handle it.
+	   The PDF interpreter extracts OutputIntent from a PDF file and sends it to here,
+	   if a device includes it in the .getdeviceparams response.
+	   This device does include due to ppm_get_params implementation.
+	   ppm_put_params must handle it (and ingore it) against 'rangecheck'.
+	 */
+	static const bool debug_print_OutputIntent = false;
+
+	if (debug_print_OutputIntent) {
+	    int i, j;
+
+	    dlprintf1("%d strings:\n", intent.size);
+	    for (i = 0; i < intent.size; i++) {
+		const gs_param_string *s = &intent.data[i];
+		dlprintf2("  %d: size %d:", i, s->size);
+		if (i < 4) {
+		    for (j = 0; j < s->size; j++)
+			dlprintf1("%c", s->data[j]);
+		} else {
+		    for (j = 0; j < s->size; j++)
+			dlprintf1(" %02x", s->data[j]);
+		}
+		dlprintf("\n");
+	    }
+	}
+    }
     save_info = pdev->color_info;
     if ((code = param_read_long(plist, (vname = "GrayValues"), &v)) != 1 ||
 	(code = param_read_long(plist, (vname = "RedValues"), &v)) != 1 ||
@@ -467,7 +526,7 @@ ppm_put_params(gx_device * pdev, gs_param_list * plist)
 
 /* Copy an alpha map, noting whether we may generate some non-black/white */
 /* colors through blending. */
-private int
+static int
 pnm_copy_alpha(gx_device * pdev, const byte * data, int data_x,
 	   int raster, gx_bitmap_id id, int x, int y, int width, int height,
 	       gx_color_index color, int depth)
@@ -486,7 +545,7 @@ pnm_copy_alpha(gx_device * pdev, const byte * data, int data_x,
 
 /* Begin processing an image, noting whether we may generate some */
 /* non-black/white colors in the process. */
-private int 
+static int 
 pnm_begin_typed_image(gx_device *dev,
                       const gs_imager_state *pis, const gs_matrix *pmat,
                       const gs_image_common_t *pim, const gs_int_rect *prect,
@@ -527,9 +586,9 @@ pnm_begin_typed_image(gx_device *dev,
 /* ------ Internal routines ------ */
 
 /* Print a page using a given row printing routine. */
-private int
+static int
 pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
-	     int (*row_proc) (P4(gx_device_printer *, byte *, int, FILE *)))
+	     int (*row_proc) (gx_device_printer *, byte *, int, FILE *))
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
     uint raster = gdev_prn_raster(pdev);
@@ -595,7 +654,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
 /* ------ Individual page printing routines ------ */
 
 /* Print a monobit page. */
-private int
+static int
 pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
@@ -617,7 +676,7 @@ pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
     }
     return 0;
 }
-private int
+static int
 pbm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -626,7 +685,7 @@ pbm_print_page(gx_device_printer * pdev, FILE * pstream)
 }
 
 /* Print a gray-mapped page. */
-private int
+static int
 pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {				/* Note that bpp <= 8 for raw format, bpp <= 16 for plain. */
@@ -636,7 +695,7 @@ pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
      * If we're writing planes for a CMYK device, we have 0 = white,
      * mask = black, which is the opposite of the pgm convention.
      */
-    uint invert = (pdev->color_info.num_components == 4 ? mask : 0);
+    uint invert = (pdev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE);
     byte *bp;
     uint x;
     int shift;
@@ -669,7 +728,7 @@ pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	}
     return 0;
 }
-private int
+static int
 pxm_pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 		  FILE * pstream)
 {				/* Compress a PGM or PPM row to a PBM row. */
@@ -713,20 +772,20 @@ pxm_pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	*dest = out;
     return pbm_print_row(pdev, data, 1, pstream);
 }
-private int
+static int
 pgm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
 
     return (bdev->uses_color == 0 && bdev->optimize ?
-	    pbm_print_page_loop(pdev, bdev->magic - 1, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 1), pstream,
 				pxm_pbm_print_row) :
 	    pbm_print_page_loop(pdev, bdev->magic, pstream,
 				pgm_print_row));
 }
 
 /* Print a color-mapped page. */
-private int
+static int
 ppgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	       FILE * pstream, bool color)
 {				/* If color=false, write only one value per pixel; */
@@ -792,19 +851,19 @@ ppgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	}
     return 0;
 }
-private int
+static int
 ppm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
     return ppgm_print_row(pdev, data, depth, pstream, true);
 }
-private int
+static int
 ppm_pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 		  FILE * pstream)
 {
     return ppgm_print_row(pdev, data, depth, pstream, false);
 }
-private int
+static int
 ppm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -813,13 +872,13 @@ ppm_print_page(gx_device_printer * pdev, FILE * pstream)
 	    pbm_print_page_loop(pdev, bdev->magic, pstream,
 				ppm_print_row) :
 	    bdev->uses_color == 1 ?
-	    pbm_print_page_loop(pdev, bdev->magic - 1, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 1), pstream,
 				ppm_pgm_print_row) :
-	    pbm_print_page_loop(pdev, bdev->magic - 2, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 2), pstream,
 				pxm_pbm_print_row));
 }
 
-private int
+static int
 pam_print_row(gx_device_printer * pdev, byte * data, int depth,
 	       FILE * pstream)
 {
@@ -828,7 +887,7 @@ pam_print_row(gx_device_printer * pdev, byte * data, int depth,
     return 0;
 }
 
-private int
+static int
 pam_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -840,7 +899,7 @@ pam_print_page(gx_device_printer * pdev, FILE * pstream)
 /* Print a faux CMYK page. */
 /* Print a row where each pixel occupies 4 bits (depth == 4). */
 /* In this case, we also know pdev->color_info.max_color == 1. */
-private int
+static int
 pkm_print_row_4(gx_device_printer * pdev, byte * data, int depth,
 		FILE * pstream)
 {
@@ -899,7 +958,7 @@ pkm_print_row_4(gx_device_printer * pdev, byte * data, int depth,
     return 0;
 }
 /* Print a row where each pixel occupies 1 or more bytes (depth >= 8). */
-private int
+static int
 pkm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
@@ -947,7 +1006,7 @@ pkm_print_row(gx_device_printer * pdev, byte * data, int depth,
     }
     return 0;
 }
-private int
+static int
 pkm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -959,7 +1018,7 @@ pkm_print_page(gx_device_printer * pdev, FILE * pstream)
 }
 
 /* Print individual separations on a single file. */
-private int
+static int
 psm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
