@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtb.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: gdevpdtb.c 8271 2007-10-07 18:09:11Z leonardo $ */
 /* BaseFont implementation for pdfwrite */
 #include "memory_.h"
 #include "ctype_.h"
@@ -62,7 +62,7 @@ pdf_has_subset_prefix(const byte *str, uint size)
     return true;
 }
 
-private inline ulong
+static inline ulong
 hash(ulong v, int index, ushort w)
 {
     return v * 3141592653u + w;
@@ -102,7 +102,7 @@ pdf_add_subset_prefix(const gx_device_pdf *pdev, gs_string *pstr, byte *used, in
 }
 
 /* Finish writing FontFile* data. */
-private int
+static int
 pdf_end_fontfile(gx_device_pdf *pdev, pdf_data_writer_t *pdw)
 {
     /* We would like to call pdf_end_data,
@@ -198,9 +198,23 @@ pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
 		goto fail;
 	}
         code = gs_copy_font_complete((gs_font *)font, complete);
-	if (code < 0)
+	if (code < 0 && pbfont->do_subset == DO_SUBSET_NO) {
+	    char buf[gs_font_name_max + 1];
+	    int l = min(copied->font_name.size, sizeof(buf) - 1);
+
+	    memcpy(buf, copied->font_name.chars, l);
+	    buf[l] = 0;
+    	    eprintf1("Can't embed the complete font %s due to font error.\n", buf);
 	    goto fail;
-	if (pbfont->num_glyphs < 0) { /* Type 1 */
+	}
+	if (code < 0) {
+	    /* A font error happened, but it may be caused by a glyph, 
+	       which is not used in the document. Continue with subsetting the font.
+	       If the failed glyph will be used in the document, 
+	       another error will hgappen when the glyph is used.
+	     */
+	    complete = copied;
+	} else if (pbfont->num_glyphs < 0) { /* Type 1 */
 	    int index, count;
 	    gs_glyph glyph;
 
@@ -378,7 +392,7 @@ pdf_write_FontFile_entry(gx_device_pdf *pdev, pdf_base_font_t *pbfont)
 /*
  * Adjust font name for Acrobat Reader 3.
  */
-private int
+static int
 pdf_adjust_font_name(gx_device_pdf *pdev, long id, pdf_base_font_t *pbfont)
 {
     /*
