@@ -1,22 +1,17 @@
-/* Copyright (C) 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevppla.c,v 1.2.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevppla.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Support for printer devices with planar buffering. */
 #include "gdevprn.h"
 #include "gdevmpla.h"
@@ -74,7 +69,7 @@ gdev_prn_put_params_planar(gx_device * pdev, gs_param_list * plist,
 }
 
 /* Set the buffer device to planar mode. */
-private int
+static int
 gdev_prn_set_planar(gx_device_memory *mdev, const gx_device *tdev)
 {
     int num_comp = tdev->color_info.num_components;
@@ -100,7 +95,7 @@ gdev_prn_set_planar(gx_device_memory *mdev, const gx_device *tdev)
 int
 gdev_prn_create_buf_planar(gx_device **pbdev, gx_device *target,
 			   const gx_render_plane_t *render_plane,
-			   gs_memory_t *mem, bool for_band)
+			   gs_memory_t *mem, gx_band_complexity_t *for_band)
 {
     int code = gx_default_create_buf_device(pbdev, target, render_plane, mem,
 					    for_band);
@@ -126,7 +121,8 @@ gdev_prn_size_buf_planar(gx_device_buf_space_t *space, gx_device *target,
 					  height, for_band);
     mdev.color_info = target->color_info;
     gdev_prn_set_planar(&mdev, target);
-    space->bits = gdev_mem_bits_size(&mdev, target->width, height);
+    if (gdev_mem_bits_size(&mdev, target->width, height, &(space->bits)) < 0)
+	return_error(gs_error_VMerror);
     space->line_ptrs = gdev_mem_line_ptrs_size(&mdev, target->width, height);
     space->raster = bitmap_raster(target->width * mdev.planes[0].depth);
     return 0;
