@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevprn.h 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: gdevprn.h 8250 2007-09-25 13:31:24Z giles $ */
 /* Common header file for memory-buffered printers */
 
 #ifndef gdevprn_INCLUDED
@@ -656,20 +656,20 @@ int gdev_prn_initialize(gx_device *, const char *, dev_proc_print_page((*)));
 void gdev_prn_init_color(gx_device *, int, dev_proc_map_rgb_color((*)), dev_proc_map_color_rgb((*)));
 
 #define prn_device_type(dtname, initproc, pageproc)\
-private dev_proc_print_page(pageproc);\
+static dev_proc_print_page(pageproc);\
 device_type(dtname, st_prn_device, initproc)
 
 #define prn_device_type_mono(dtname, dname, initproc, pageproc)\
-private dev_proc_print_page(pageproc);\
-private int \
+static dev_proc_print_page(pageproc);\
+static int \
 initproc(gx_device *dev)\
 {	return gdev_prn_initialize(dev, dname, pageproc);\
 }\
 device_type(dtname, st_prn_device, initproc)
 
 #define prn_device_type_color(dtname, dname, depth, initproc, pageproc, rcproc, crproc)\
-private dev_proc_print_page(pageproc);\
-private int \
+static dev_proc_print_page(pageproc);\
+static int \
 initproc(gx_device *dev)\
 {	int code = gdev_prn_initialize(dev, dname, pageproc);\
 	gdev_prn_init_color(dev, depth, rcproc, crproc);\
