@@ -1,22 +1,17 @@
-/* Copyright (C) 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevprna.c,v 1.2.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevprna.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Generic asynchronous printer driver support */
 
 /* Initial version 2/1/98 by John Desrosiers (soho@crl.com) */
@@ -48,24 +43,24 @@
 #endif /* defined(DEBUG) */
 
 /* ---------------- Standard device procedures ---------------- */
-private dev_proc_close_device(gdev_prn_async_write_close_device);
-private dev_proc_output_page(gdev_prn_async_write_output_page);
-private dev_proc_put_params(gdev_prn_async_write_put_params);
-private dev_proc_get_hardware_params(gdev_prn_async_write_get_hardware_params);
-private dev_proc_put_params(gdev_prn_async_render_put_params);
+static dev_proc_close_device(gdev_prn_async_write_close_device);
+static dev_proc_output_page(gdev_prn_async_write_output_page);
+static dev_proc_put_params(gdev_prn_async_write_put_params);
+static dev_proc_get_hardware_params(gdev_prn_async_write_get_hardware_params);
+static dev_proc_put_params(gdev_prn_async_render_put_params);
 
 /* ---------------- Forward Declarations ---------------------- */
-private void gdev_prn_dealloc(P1(gx_device_printer *));
-private proc_free_up_bandlist_memory(gdev_prn_async_write_free_up_bandlist_memory);
-private int flush_page(P2(gx_device_printer *, bool));
-private int reopen_clist_after_flush(P1(gx_device_printer *));
-private void reinit_printer_into_printera(P1(gx_device_printer * const));
-private int alloc_bandlist_memory(P2(gs_memory_t **, gs_memory_t *));
-private void free_bandlist_memory(P1(gs_memory_t *));
-private int alloc_render_memory(P3(gs_memory_t **, gs_memory_t *, long));
-private void free_render_memory(P1(gs_memory_t *));
-private gs_memory_recover_status_t
-    prna_mem_recover(P2(gs_memory_retrying_t *rmem, void *proc_data));
+static void gdev_prn_dealloc(gx_device_printer *);
+static proc_free_up_bandlist_memory(gdev_prn_async_write_free_up_bandlist_memory);
+static int flush_page(gx_device_printer *, bool);
+static int reopen_clist_after_flush(gx_device_printer *);
+static void reinit_printer_into_printera(gx_device_printer * const);
+static int alloc_bandlist_memory(gs_memory_t **, gs_memory_t *);
+static void free_bandlist_memory(gs_memory_t *);
+static int alloc_render_memory(gs_memory_t **, gs_memory_t *, long);
+static void free_render_memory(gs_memory_t *);
+static gs_memory_recover_status_t
+    prna_mem_recover(gs_memory_retrying_t *rmem, void *proc_data);
 
 /* ------ Open/close ------ */
 
@@ -101,7 +96,7 @@ gdev_prn_async_write_open(gx_device_printer * pwdev, int max_raster,
     /* The * 2's in the next statement are a ****** HACK ****** to deal with */
     /* sandbars in the memory manager. */
     if ((code = alloc_render_memory(&render_memory,
-	    &gs_memory_default, RendererAllocationOverheadBytes + max_raster
+	    pwdev->memory->non_gc_memory, RendererAllocationOverheadBytes + max_raster
 				    /* the first * 2 is not a hack */
 		   + (max_raster + sizeof(void *) * 2) * min_band_height
 		   + max_src_image_row + gx_ht_cache_default_bits() * 2)) < 0)
@@ -111,7 +106,7 @@ gdev_prn_async_write_open(gx_device_printer * pwdev, int max_raster,
     /* Bandlist mem is threadsafe & common to rdr/wtr, so it's used */
     /* for page queue & cmd list buffers. */
     if ((code = alloc_bandlist_memory
-	 (&pwdev->bandlist_memory, &gs_memory_default)) < 0)
+	 (&pwdev->bandlist_memory, pwdev->memory->non_gc_memory)) < 0)
 	goto open_err;
 
     /* Dictate banding parameters for both renderer & writer */
@@ -175,7 +170,7 @@ gdev_prn_async_write_open(gx_device_printer * pwdev, int max_raster,
 	prdev->page_queue = pwdev->page_queue;
 
 	/* Start renderer thread & wait for its successful open of device */
-	if (!(open_semaphore = gx_semaphore_alloc(&gs_memory_default)))
+	if (!(open_semaphore = gx_semaphore_alloc(prdev->memory)))
 	    code = gs_note_error(gs_error_VMerror);
 	else {
 	    gdev_prn_start_render_params thread_params;
@@ -194,7 +189,7 @@ gdev_prn_async_write_open(gx_device_printer * pwdev, int max_raster,
     /* ----- Set the recovery procedure for the mem allocator ----- */
     if (code >= 0) {
 	gs_memory_retrying_set_recover(
-		(gs_memory_retrying_t *)&gs_memory_default,
+		(gs_memory_retrying_t *)pwdev->memory->non_gc_memory,
 		prna_mem_recover,
 		(void *)pcwdev
 	    );
@@ -219,7 +214,7 @@ open_err:
 /* This procedure is called from within the memory allocator when regular */
 /* malloc's fail -- this procedure tries to free up pages from the queue  */
 /* and returns a status code indicating whether any more can be freed.    */
-private gs_memory_recover_status_t
+static gs_memory_recover_status_t
 prna_mem_recover(gs_memory_retrying_t *rmem, void *proc_data)
 {
     int pages_remain = 0;
@@ -232,7 +227,7 @@ prna_mem_recover(gs_memory_retrying_t *rmem, void *proc_data)
 }
 
 /* (Re)set printer device fields which get trampled by gdevprn_open & put_params */
-private void
+static void
 reinit_printer_into_printera(
 			     gx_device_printer * const pdev	/* printer to convert */
 )
@@ -252,13 +247,15 @@ reinit_printer_into_printera(
 }
 
 /* Generic closing for the writer device. */
-private int
+static int
 gdev_prn_async_write_close_device(gx_device * pdev)
 {
     gx_device_printer *const pwdev = (gx_device_printer *) pdev;
+    gx_device_clist_writer *const pcwdev =
+	&((gx_device_clist *) pdev)->writer;
 
     /* Signal render thread to close & terminate when done */
-    gx_page_queue_add_page(pwdev->page_queue,
+    gx_page_queue_add_page(pcwdev, pwdev->page_queue,
 			   GX_PAGE_QUEUE_ACTION_TERMINATE, 0, 0);
 
     /* Wait for renderer to finish all pages & terminate req */
@@ -274,7 +271,7 @@ gdev_prn_async_write_close_device(gx_device * pdev)
 }
 
 /* Deallocte dynamic memory attached to device. Aware of possible imcomplete open */
-private void
+static void
 gdev_prn_dealloc(gx_device_printer * pwdev)
 {
     gx_device_printer *const prdev = pwdev->async_renderer;
@@ -322,7 +319,7 @@ gdev_prn_async_render_close_device(gx_device_printer * prdev)
 }
 
 /* (Re)set renderer device fields which get trampled by gdevprn_open & put_params */
-private void
+static void
 reinit_printer_into_renderer(
 			     gx_device_printer * const pdev	/* printer to convert */
 )
@@ -445,7 +442,7 @@ gdev_prn_async_render_thread(
 /* ------ Get/put parameters ------ */
 
 /* Put parameters. */
-private int
+static int
 gdev_prn_async_write_put_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_clist_writer *const pclwdev =
@@ -539,7 +536,7 @@ gdev_prn_async_write_put_params(gx_device * pdev, gs_param_list * plist)
 }
 
 /* Get hardware-detected params. Drain page queue, then call renderer version */
-private int
+static int
 gdev_prn_async_write_get_hardware_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_printer *const pwdev = (gx_device_printer *) pdev;
@@ -559,7 +556,7 @@ gdev_prn_async_write_get_hardware_params(gx_device * pdev, gs_param_list * plist
 }
 
 /* Put parameters on RENDERER. */
-private int		/* returns -ve err code only if FATAL error (can't keep rendering) */
+static int		/* returns -ve err code only if FATAL error (can't keep rendering) */
 gdev_prn_async_render_put_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_printer *const prdev = (gx_device_printer *) pdev;
@@ -591,7 +588,7 @@ gdev_prn_async_render_put_params(gx_device * pdev, gs_param_list * plist)
 /* ------ Others ------ */
 
 /* Output page causes file to get added to page queue for later rasterizing */
-private int
+static int
 gdev_prn_async_write_output_page(gx_device * pdev, int num_copies, int flush)
 {
     gx_device_printer *const pwdev = (gx_device_printer *) pdev;
@@ -604,7 +601,7 @@ gdev_prn_async_write_output_page(gx_device * pdev, int num_copies, int flush)
 
     /* do NOT close files before sending to page queue */
     flush_code = clist_end_page(pcwdev);
-    add_code = gx_page_queue_add_page(pwdev->page_queue,
+    add_code = gx_page_queue_add_page(pcwdev, pwdev->page_queue,
 				(flush ? GX_PAGE_QUEUE_ACTION_FULL_PAGE :
 				 GX_PAGE_QUEUE_ACTION_COPY_PAGE),
 				      &pcwdev->page_info, num_copies);
@@ -628,7 +625,7 @@ gdev_prn_async_write_output_page(gx_device * pdev, int num_copies, int flush)
 }
 
 /* Free bandlist memory waits until the rasterizer runs enough to free some mem */
-private int			/* -ve err,  0 if no pages remain to rasterize, 1 if more pages to go */
+static int			/* -ve err,  0 if no pages remain to rasterize, 1 if more pages to go */
 gdev_prn_async_write_free_up_bandlist_memory(gx_device * pdev, bool flush_current)
 {
     gx_device_printer *const pwdev = (gx_device_printer *) pdev;
@@ -646,7 +643,7 @@ gdev_prn_async_write_free_up_bandlist_memory(gx_device * pdev, bool flush_curren
 
 /* Flush out any partial pages accumulated in device */
 /* LEAVE DEVICE in a state where it must be re-opened/re-init'd */
-private int			/* ret 0 ok no flush, -ve error code */
+static int			/* ret 0 ok no flush, -ve error code */
 flush_page(
 	   gx_device_printer * pwdev,	/* async writer device to flush */
 	   bool partial	/* true if only partial page */
@@ -659,7 +656,7 @@ flush_page(
 
     /* do NOT close files before sending to page queue */
     flush_code = clist_end_page(pcwdev);
-    add_code = gx_page_queue_add_page(pwdev->page_queue,
+    add_code = gx_page_queue_add_page(pcwdev, pwdev->page_queue,
 				(partial ? GX_PAGE_QUEUE_ACTION_PARTIAL_PAGE :
 				 GX_PAGE_QUEUE_ACTION_FULL_PAGE),
 				      &pcwdev->page_info, 0);
@@ -674,7 +671,7 @@ flush_page(
 }
 
 /* Flush any pending partial pages, re-open device */
-private int
+static int
 reopen_clist_after_flush(
 			 gx_device_printer * pwdev	/* async writer device to flush */
 )
@@ -703,7 +700,7 @@ reopen_clist_after_flush(
  */
 
 /* Create a bandlist allocator. */
-private int
+static int
 alloc_bandlist_memory(gs_memory_t ** final_allocator,
 		      gs_memory_t * base_allocator)
 {
@@ -741,7 +738,7 @@ alloc_err:
 }
 
 /* Free a bandlist allocator. */
-private void
+static void
 free_bandlist_memory(gs_memory_t *bandlist_allocator)
 {
     gs_memory_locked_t *const lmem = (gs_memory_locked_t *)bandlist_allocator;
@@ -756,12 +753,12 @@ free_bandlist_memory(gs_memory_t *bandlist_allocator)
 }
 
 /* Create an allocator with a fixed memory limit. */
-private int
+static int
 alloc_render_memory(gs_memory_t **final_allocator,
 		    gs_memory_t *base_allocator, long space)
 {
     gs_ref_memory_t *rmem =
-	ialloc_alloc_state((gs_raw_memory_t *)base_allocator, space);
+	ialloc_alloc_state((gs_memory_t *)base_allocator, space);
     vm_spaces spaces;
     int i, code;
 
@@ -788,7 +785,7 @@ alloc_render_memory(gs_memory_t **final_allocator,
 }
 
 /* Free an allocator with a fixed memory limit. */
-private void
+static void
 free_render_memory(gs_memory_t *render_allocator)
 {
     if (render_allocator)
