@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsalphac.c,v 1.2.6.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gsalphac.c 8334 2007-10-30 17:47:49Z leonardo $ */
 /* Alpha-compositing implementation */
 #include "memory_.h"
 #include "gx.h"
@@ -64,10 +59,10 @@ typedef struct const_pixel_row_s {
  * alpha data to test whether there are any actual values that would
  * generate a non-unity alpha result.
  */
-int composite_values(P5(const pixel_row_t * pdest,
-			const const_pixel_row_t * psource,
-			int values_per_pixel, uint num_pixels,
-			const gs_composite_params_t * pcp));
+int composite_values(const pixel_row_t * pdest,
+		     const const_pixel_row_t * psource,
+		     int values_per_pixel, uint num_pixels,
+		     const gs_composite_params_t * pcp);
 
 /* ---------------- Alpha-compositing objects ---------------- */
 
@@ -93,17 +88,20 @@ int composite_values(P5(const pixel_row_t * pdest,
 /* ------ Object definition and creation ------ */
 
 /* Define alpha-compositing objects. */
-private composite_create_default_compositor_proc(c_alpha_create_default_compositor);
-private composite_equal_proc(c_alpha_equal);
-private composite_write_proc(c_alpha_write);
-private composite_read_proc(c_alpha_read);
-private const gs_composite_type_t gs_composite_alpha_type =
+static composite_create_default_compositor_proc(c_alpha_create_default_compositor);
+static composite_equal_proc(c_alpha_equal);
+static composite_write_proc(c_alpha_write);
+static composite_read_proc(c_alpha_read);
+const gs_composite_type_t gs_composite_alpha_type =
 {
+    GX_COMPOSITOR_ALPHA,
     {
 	c_alpha_create_default_compositor,
 	c_alpha_equal,
 	c_alpha_write,
-	c_alpha_read
+	c_alpha_read,
+	gx_default_composite_clist_write_update,
+	gx_default_composite_clist_read_update
     }
 };
 typedef struct gs_composite_alpha_s {
@@ -125,7 +123,7 @@ gs_create_composite_alpha(gs_composite_t ** ppcte,
 		      mem, return_error(gs_error_VMerror),
 		      "gs_create_composite_alpha");
     pcte->type = &gs_composite_alpha_type;
-    pcte->id = gs_next_ids(1);
+    pcte->id = gs_next_ids(mem, 1);
     pcte->params = *params;
     *ppcte = (gs_composite_t *) pcte;
     return 0;
@@ -135,7 +133,7 @@ gs_create_composite_alpha(gs_composite_t ** ppcte,
 
 #define pacte ((const gs_composite_alpha_t *)pcte)
 
-private bool
+static bool
 c_alpha_equal(const gs_composite_t * pcte, const gs_composite_t * pcte2)
 {
     return (pcte2->type == pcte->type &&
@@ -146,12 +144,13 @@ c_alpha_equal(const gs_composite_t * pcte, const gs_composite_t * pcte2)
 #undef pacte2
 }
 
-private int
+static int
 c_alpha_write(const gs_composite_t * pcte, byte * data, uint * psize)
 {
     uint size = *psize;
     uint used;
 
+    if_debug1('v', "[v]c_alpha_write(%d)\n", pacte->params.op);
     if (pacte->params.op == composite_Dissolve) {
 	used = 1 + sizeof(pacte->params.delta);
 	if (size < used) {
@@ -171,24 +170,25 @@ c_alpha_write(const gs_composite_t * pcte, byte * data, uint * psize)
     return 0;
 }
 
-private int
+static int
 c_alpha_read(gs_composite_t ** ppcte, const byte * data, uint size,
 	     gs_memory_t * mem)
 {
     gs_composite_alpha_params_t params;
+    int code, nbytes = 1;
 
     if (size < 1 || *data > composite_op_last)
 	return_error(gs_error_rangecheck);
     params.op = *data;
+    if_debug1('v', "[v]c_alpha_read(%d)\n", params.op);
     if (params.op == composite_Dissolve) {
-	if (size != 1 + sizeof(params.delta))
+	if (size < 1 + sizeof(params.delta))
 	    return_error(gs_error_rangecheck);
 	memcpy(&params.delta, data + 1, sizeof(params.delta));
-    } else {
-	if (size != 1)
-	    return_error(gs_error_rangecheck);
+	nbytes += sizeof(params.delta);
     }
-    return gs_create_composite_alpha(ppcte, &params, mem);
+    code = gs_create_composite_alpha(ppcte, &params, mem);
+    return code < 0 ? code : nbytes;
 }
 
 /* ---------------- Alpha-compositing device ---------------- */
@@ -204,16 +204,16 @@ gs_private_st_suffix_add0_final(st_device_composite_alpha,
     device_c_alpha_enum_ptrs, device_c_alpha_reloc_ptrs, gx_device_finalize,
 				st_device_forward);
 /* The device descriptor. */
-private dev_proc_close_device(dca_close);
-private dev_proc_fill_rectangle(dca_fill_rectangle);
-private dev_proc_map_rgb_color(dca_map_rgb_color);
-private dev_proc_map_color_rgb(dca_map_color_rgb);
-private dev_proc_copy_mono(dca_copy_mono);
-private dev_proc_copy_color(dca_copy_color);
-private dev_proc_map_rgb_alpha_color(dca_map_rgb_alpha_color);
-private dev_proc_map_color_rgb_alpha(dca_map_color_rgb_alpha);
-private dev_proc_copy_alpha(dca_copy_alpha);
-private const gx_device_composite_alpha gs_composite_alpha_device =
+static dev_proc_close_device(dca_close);
+static dev_proc_fill_rectangle(dca_fill_rectangle);
+static dev_proc_map_rgb_color(dca_map_rgb_color);
+static dev_proc_map_color_rgb(dca_map_color_rgb);
+static dev_proc_copy_mono(dca_copy_mono);
+static dev_proc_copy_color(dca_copy_color);
+static dev_proc_map_rgb_alpha_color(dca_map_rgb_alpha_color);
+static dev_proc_map_color_rgb_alpha(dca_map_color_rgb_alpha);
+static dev_proc_copy_alpha(dca_copy_alpha);
+static const gx_device_composite_alpha gs_composite_alpha_device =
 {std_device_std_body_open(gx_device_composite_alpha, 0,
 			  "alpha compositor", 0, 0, 1, 1),
  {gx_default_open_device,
@@ -261,10 +261,10 @@ private const gx_device_composite_alpha gs_composite_alpha_device =
 };
 
 /* Create an alpha compositor. */
-private int
+static int
 c_alpha_create_default_compositor(const gs_composite_t * pcte,
-	   gx_device ** pcdev, gx_device * dev, const gs_imager_state * pis,
-				  gs_memory_t * mem)
+	   gx_device ** pcdev, gx_device * dev, gs_imager_state * pis,
+	   gs_memory_t * mem)
 {
     gx_device_composite_alpha *cdev;
 
@@ -305,7 +305,7 @@ c_alpha_create_default_compositor(const gs_composite_t * pcte,
 }
 
 /* Close the device and free its storage. */
-private int
+static int
 dca_close(gx_device * dev)
 {				/*
 				 * Finalization will call close again: avoid a recursion loop.
@@ -317,13 +317,12 @@ dca_close(gx_device * dev)
 
 /* ------ (RGB) color mapping ------ */
 
-private gx_color_index
-dca_map_rgb_color(gx_device * dev,
-		  gx_color_value r, gx_color_value g, gx_color_value b)
+static gx_color_index
+dca_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-    return dca_map_rgb_alpha_color(dev, r, g, b, gx_max_color_value);
+    return dca_map_rgb_alpha_color(dev, cv[0], cv[1], cv[2], gx_max_color_value);
 }
-private gx_color_index
+static gx_color_index
 dca_map_rgb_alpha_color(gx_device * dev,
 	      gx_color_value red, gx_color_value green, gx_color_value blue,
 			gx_color_value alpha)
@@ -367,7 +366,7 @@ dca_map_rgb_alpha_color(gx_device * dev,
 #undef premult
     return (color << 8) + a;
 }
-private int
+static int
 dca_map_color_rgb(gx_device * dev, gx_color_index color,
 		  gx_color_value prgb[3])
 {
@@ -414,7 +413,7 @@ dca_map_color_rgb(gx_device * dev, gx_color_index color,
 #undef postdiv
     return 0;
 }
-private int
+static int
 dca_map_color_rgb_alpha(gx_device * dev, gx_color_index color,
 			gx_color_value prgba[4])
 {
@@ -424,7 +423,7 @@ dca_map_color_rgb_alpha(gx_device * dev, gx_color_index color,
 
 /* ------ Imaging ------ */
 
-private int
+static int
 dca_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 		   gx_color_index color)
 {				/* This is where all the real work gets done! */
@@ -453,7 +452,7 @@ dca_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     }
     rect.p.x = x, rect.q.x = x + w;
     std_params.options =
-	gb_colors_for_device(dev) |
+	GB_COLORS_NATIVE |
 	(GB_ALPHA_LAST | GB_DEPTH_8 | GB_PACKING_CHUNKY |
 	 GB_RETURN_COPY | GB_RETURN_POINTER | GB_ALIGN_ANY |
 	 GB_OFFSET_0 | GB_OFFSET_ANY | GB_RASTER_STANDARD |
@@ -519,7 +518,7 @@ dca_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     return code;
 }
 
-private int
+static int
 dca_copy_mono(gx_device * dev, const byte * data,
 	    int dx, int raster, gx_bitmap_id id, int x, int y, int w, int h,
 	      gx_color_index zero, gx_color_index one)
@@ -529,7 +528,7 @@ dca_copy_mono(gx_device * dev, const byte * data,
 				zero, one);
 }
 
-private int
+static int
 dca_copy_color(gx_device * dev, const byte * data,
 	       int dx, int raster, gx_bitmap_id id,
 	       int x, int y, int w, int h)
@@ -538,7 +537,7 @@ dca_copy_color(gx_device * dev, const byte * data,
     return gx_default_copy_color(dev, data, dx, raster, id, x, y, w, h);
 }
 
-private int
+static int
 dca_copy_alpha(gx_device * dev, const byte * data, int data_x,
 	   int raster, gx_bitmap_id id, int x, int y, int width, int height,
 	       gx_color_index color, int depth)
