@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscdef.c 8160 2007-08-01 19:18:12Z giles $ */
+/* $Id: gscdef.c 8388 2007-11-19 23:57:46Z giles $ */
 /* Configuration scalars */
 
 #include "std.h"
@@ -83,3 +83,9 @@ const char *const gs_lib_default_path = GS_LIB_DEFAULT;
 
 /* Define the interpreter initialization file. */
 const char *const gs_init_file = GS_INIT;
+
+/* Define the default devices list. */
+#ifndef GS_DEV_DEFAULT
+#define GS_DEV_DEFAULT ""
+#endif
+const char *const gs_dev_defaults = GS_DEV_DEFAULT;
