@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscie.h 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: gscie.h 8192 2007-08-17 04:49:22Z ray $ */
 /* Structures for CIE color algorithms */
 /* (requires gscspace.h, gscolor2.h) */
 
@@ -140,6 +140,7 @@ typedef long cie_cached_value;
 		    _cie_fixed_shift * 2 - _cie_product_excess_bits - (fbits)) :\
       arith_rshift((v) * (factor), _cie_fixed_shift * 2 - (fbits)))
 #  define cie_cached_rshift(v, n) arith_rshift(v, n)
+#  define cie_cached_abs(v) any_abs(v)  /* labs() is C89 extension */
 #else
 typedef float cie_cached_value;
 #  define float2cie_cached(v) (v)
@@ -149,6 +150,7 @@ typedef float cie_cached_value;
 #  define cie_cached_product2int(v, factor, fbits)\
      ((int)float_lshift((v) * (factor), fbits))
 #  define cie_cached_rshift(v, n) float_rshift(v, n)
+#  define cie_cached_abs(v) fabs(v)  /* intristic on MSVC and GCC */
 #endif
 
 /* ---------------- Structures ---------------- */
@@ -684,13 +686,38 @@ gx_cie_joint_caches *gx_unshare_cie_caches(gs_state *);
 gx_cie_joint_caches *gx_currentciecaches(gs_state *);
 const gs_cie_common *gs_cie_cs_common(const gs_state *);
 int gs_cie_cs_complete(gs_state *, bool);
-int gs_cie_jc_complete(const gs_imager_state *, const gs_color_space *pcs);
+int gs_cie_jc_complete(const gs_imager_state *, const gs_color_space *);
 float gs_cie_cached_value(floatp, const cie_cache_floats *);
+int gx_install_cie_abc(gs_cie_abc *, gs_state *);
 
 #define CIE_CLAMP_INDEX(index)\
   index = (index < 0 ? 0 :\
 	   index >= gx_cie_cache_size ? gx_cie_cache_size - 1 : index)
 
+/* Define the template for loading a cache. */
+/* If we had parameterized types, or a more flexible type system, */
+/* this could be done with a single procedure. */
+#define CIE_LOAD_CACHE_BODY(pcache, domains, rprocs, dprocs, pcie, cname)\
+  BEGIN\
+	int j;\
+\
+	for (j = 0; j < countof(pcache); j++) {\
+	    cie_cache_floats *pcf = &(pcache)[j].floats;\
+	    int i;\
+	    gs_sample_loop_params_t lp;\
+\
+	    gs_cie_cache_init(&pcf->params, &lp, &(domains)[j], cname);\
+	    for (i = 0; i <= lp.N; ++i) {\
+		float v = SAMPLE_LOOP_VALUE(i, lp);\
+		pcf->values[i] = (*(rprocs)->procs[j])(v, pcie);\
+		if_debug5('C', "[C]%s[%d,%d] = %g => %g\n",\
+			  cname, j, i, v, pcf->values[i]);\
+	    }\
+	    pcf->params.is_identity =\
+		(rprocs)->procs[j] == (dprocs).procs[j];\
+	}\
+  END
+
 /*
  * Compute the source and destination WhitePoint and BlackPoint for
  * the TransformPQR procedure.
