@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsdparam.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsdparam.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -29,7 +29,7 @@
 /* ================ Getting parameters ================ */
 
 /* Forward references */
-private bool param_HWColorMap(gx_device *, byte *);
+static bool param_HWColorMap(gx_device *, byte *);
 
 /* Get the device parameters. */
 int
@@ -117,12 +117,6 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     /* Transmit the values. */
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-    /* The named color callback pointer */
-    code = named_color_callback_get_params(dev, plist);
-    if (code < 0)
-	return code;
-#endif
     if (
 
 	/* Standard parameters */
@@ -168,6 +162,14 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 	)
 	return code;
 
+    /* If LeadingEdge was set explicitly, report it here. */
+    if (dev->LeadingEdge & LEADINGEDGE_SET_MASK) {
+	int leadingedge = dev->LeadingEdge & LEADINGEDGE_MASK;
+	code = param_write_int(plist, "LeadingEdge", &leadingedge);
+    }
+    if (code < 0)
+	return code;
+
     /* Fill in color information. */
 
     if (colors > 1) {
@@ -199,7 +201,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 }
 
 /* Get the color map for a device.  Return true if there is one. */
-private bool
+static bool
 param_HWColorMap(gx_device * dev, byte * palette /* 3 << 8 */ )
 {
     int depth = dev->color_info.depth;
@@ -234,7 +236,7 @@ gx_default_get_hardware_params(gx_device * dev, gs_param_list * plist)
 /* ---------------- Input and output media ---------------- */
 
 /* Finish defining input or output media. */
-private int
+static int
 finish_media(gs_param_list * mlist, gs_param_name key, const char *media_type)
 {
     int code = 0;
@@ -380,15 +382,16 @@ gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict)
 /* ================ Putting parameters ================ */
 
 /* Forward references */
-private int param_anti_alias_bits(gs_param_list *, gs_param_name, int *);
-private int param_MediaSize(gs_param_list *, gs_param_name,
+static int param_normalize_anti_alias_bits( uint max_gray, int bits );
+static int param_anti_alias_bits(gs_param_list *, gs_param_name, int *);
+static int param_MediaSize(gs_param_list *, gs_param_name,
 			    const float *, gs_param_float_array *);
 
-private int param_check_bool(gs_param_list *, gs_param_name, bool, bool);
-private int param_check_long(gs_param_list *, gs_param_name, long, bool);
+static int param_check_bool(gs_param_list *, gs_param_name, bool, bool);
+static int param_check_long(gs_param_list *, gs_param_name, long, bool);
 #define param_check_int(plist, pname, ival, is_defined)\
   param_check_long(plist, pname, (long)(ival), is_defined)
-private int param_check_bytes(gs_param_list *, gs_param_name, const byte *,
+static int param_check_bytes(gs_param_list *, gs_param_name, const byte *,
 			      uint, bool);
 #define param_check_string(plist, pname, str, is_defined)\
   param_check_bytes(plist, pname, (const byte *)(str), \
@@ -443,6 +446,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     int tab = dev->color_info.anti_alias.text_bits;
     int gab = dev->color_info.anti_alias.graphics_bits;
     gs_param_string cms;
+    int leadingedge = dev->LeadingEdge;
 
     /*
      * Template:
@@ -472,12 +476,17 @@ e:	param_signal_error(plist, param_name, ecode);\
     }\
     END
 
-#if ENABLE_NAMED_COLOR_CALLBACK
-    /* The named_color callback pointer */
-    code = named_color_callback_put_params(dev, plist);
-    if (code < 0)
-	ecode = code;
-#endif
+    /*
+     * The actual value of LeadingEdge must be changed inside this routine,
+     * so that we can detect that it has been changed. Thus, instead of a
+     * device setting the value itself, it signals a request, which is
+     * now executed.
+     */
+    if (leadingedge & LEADINGEDGE_REQ_BIT) {
+	leadingedge = (leadingedge & LEADINGEDGE_SET_MASK) |
+	    ((leadingedge >> LEADINGEDGE_REQ_VAL_SHIFT) & LEADINGEDGE_MASK);
+    }
+
     /*
      * The HWResolution, HWSize, and MediaSize parameters interact in
      * the following way:
@@ -488,6 +497,13 @@ e:	param_signal_error(plist, param_name, ecode);\
      * in the order 1, 2, 3.  This does the right thing in the most
      * common case of setting more than one parameter, namely,
      * setting both HWResolution and HWSize.
+     *
+     * Changing of LeadingEdge is treated exactly the same as a
+     * change in HWResolution. In typical usage, MediaSize is
+     * short-edge (MediaSize[0] < MediaSize[1]), so if LeadingEdge
+     * is 1 or 3, then HWSize will become long-edge. For nonsquare
+     * resolutions, HWResolution[0] always corresponds with width
+     * (scan length), and [1] with height (number of scans).
      */
 
     BEGIN_ARRAY_PARAM(param_read_float_array, "HWResolution", hwra, 2, hwre) {
@@ -512,6 +528,26 @@ e:	param_signal_error(plist, param_name, ecode);\
 	else
 	    break;
     } END_ARRAY_PARAM(hwsa, hwse);
+    {
+	int t;
+
+	code = param_read_int(plist, "LeadingEdge", &t);
+	if (code < 0) {
+	    if (param_read_null(plist, "LeadingEdge") == 0) {
+		/* if param is null, clear explicitly-set flag */
+		leadingedge &= ~LEADINGEDGE_SET_MASK;
+		code = 0;
+	    } else {
+		ecode = code;
+	    }
+	} else if (code == 0) {
+	    if (t < 0 || t > 3)
+		param_signal_error(plist, "LeadingEdge",
+				   ecode = gs_error_rangecheck);
+	    else
+		leadingedge = LEADINGEDGE_SET_MASK | t;
+	}
+    }
     {
 	const float *res = (hwra.data == 0 ? dev->HWResolution : hwra.data);
 
@@ -699,12 +735,14 @@ nce:
     if (code < 0)
 	return code;
 
-    /* Now actually make the changes. */
-    /* Changing resolution or page size requires closing the device, */
-    /* but changing margins or ImagingBBox does not. */
-    /* In order not to close and reopen the device unnecessarily, */
-    /* we check for replacing the values with the same ones. */
-
+    /* 
+     * Now actually make the changes. Changing resolution, rotation
+     * (through LeadingEdge) or page size requires closing the device,
+     * but changing margins or ImagingBBox does not. In order not to
+     * close and reopen the device unnecessarily, we check for
+     * replacing the values with the same ones.
+     */
+    
     if (hwra.data != 0 &&
 	(dev->HWResolution[0] != hwra.data[0] ||
 	 dev->HWResolution[1] != hwra.data[1])
@@ -713,6 +751,19 @@ nce:
 	    gs_closedevice(dev);
 	gx_device_set_resolution(dev, hwra.data[0], hwra.data[1]);
     }
+    if ((leadingedge & LEADINGEDGE_MASK) !=
+	(dev->LeadingEdge & LEADINGEDGE_MASK)) {
+	/* If the LeadingEdge_set flag changes but the value of LeadingEdge
+	   itself does not, don't close device and recompute page size. */
+	dev->LeadingEdge = leadingedge;
+	if (dev->is_open)
+	    gs_closedevice(dev);
+	gx_device_set_resolution(dev, dev->HWResolution[0], dev->HWResolution[1]);
+    }
+    /* clear leadingedge request, preserve "set" flag */
+    dev->LeadingEdge &= LEADINGEDGE_MASK;
+    dev->LeadingEdge |= (leadingedge & LEADINGEDGE_SET_MASK);
+
     if (hwsa.data != 0 &&
 	(dev->width != hwsa.data[0] ||
 	 dev->height != hwsa.data[1])
@@ -752,15 +803,38 @@ nce:
 	dev->ImagingBBox_set = false;
     }
     dev->UseCIEColor = ucc;
-    dev->color_info.anti_alias.text_bits = tab;
-    dev->color_info.anti_alias.graphics_bits = gab;
+	dev->color_info.anti_alias.text_bits =
+		param_normalize_anti_alias_bits(max(dev->color_info.max_gray,
+			dev->color_info.max_color), tab);
+	dev->color_info.anti_alias.graphics_bits =
+		param_normalize_anti_alias_bits(max(dev->color_info.max_gray,
+			dev->color_info.max_color), gab);
     dev->LockSafetyParams = locksafe;
     gx_device_decache_colors(dev);
     return 0;
 }
 
+void
+gx_device_request_leadingedge(gx_device *dev, int le_req)
+{
+    dev->LeadingEdge = (dev->LeadingEdge & ~LEADINGEDGE_REQ_VAL) |
+	((le_req << LEADINGEDGE_REQ_VAL_SHIFT) & LEADINGEDGE_REQ_VAL) |
+	LEADINGEDGE_REQ_BIT;
+}
+
+/* Limit the anti-alias bit values to the maximum legal value for the
+ * current color depth.
+ */
+static int
+param_normalize_anti_alias_bits( uint max_gray, int bits )
+{
+	int	max_bits = ilog2( max_gray + 1);
+	
+	return  (bits > max_bits ? max_bits : bits);
+}
+
 /* Read TextAlphaBits or GraphicsAlphaBits. */
-private int
+static int
 param_anti_alias_bits(gs_param_list * plist, gs_param_name param_name, int *pa)
 {
     int code = param_read_int(plist, param_name, pa);
@@ -783,7 +857,7 @@ param_anti_alias_bits(gs_param_list * plist, gs_param_name param_name, int *pa)
 
 
 /* Read .MediaSize or, if supported as a synonym, PageSize. */
-private int
+static int
 param_MediaSize(gs_param_list * plist, gs_param_name pname,
 		const float *res, gs_param_float_array * pa)
 {
@@ -811,7 +885,7 @@ param_MediaSize(gs_param_list * plist, gs_param_name pname,
 
 /* Check that a nominally read-only parameter is being set to */
 /* its existing value. */
-private int
+static int
 param_check_bool(gs_param_list * plist, gs_param_name pname, bool value,
 		 bool is_defined)
 {
@@ -833,7 +907,7 @@ param_check_bool(gs_param_list * plist, gs_param_name pname, bool value,
     }
     return code;
 }
-private int
+static int
 param_check_long(gs_param_list * plist, gs_param_name pname, long value,
 		 bool is_defined)
 {
@@ -855,7 +929,7 @@ param_check_long(gs_param_list * plist, gs_param_name pname, long value,
     }
     return code;
 }
-private int
+static int
 param_check_bytes(gs_param_list * plist, gs_param_name pname, const byte * str,
 		  uint size, bool is_defined)
 {
