@@ -1,24 +1,29 @@
-/* Copyright (C) 1989, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsmisc.c,v 1.12.2.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gsmisc.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Miscellaneous utilities for Ghostscript library */
 
+
+/****************************************************************************
+ * This global should be put into either gs_imager_state or gs_device,
+ * but the "plumbing" retrofit is more than we need for things that were
+ * previously compile time modifications. CPSI_mode will at least make these
+ * dynamic (but NOT thread safe)!!!
+ ****************************************************************************
+ */
+int CPSI_mode = 0; /* false */	/* default GS behavior is 'false' */
+
 /*
  * In order to capture the original definition of sqrt, which might be
  * either a procedure or a macro and might not have an ANSI-compliant
@@ -31,7 +36,7 @@
 #else
 #  include <math.h>
 #endif
-inline private double
+static inline double
 orig_sqrt(double x)
 {
     return sqrt(x);
@@ -50,17 +55,14 @@ orig_sqrt(double x)
 #include "gconfigv.h"		/* for USE_ASM */
 #include "gxfarith.h"
 #include "gxfixed.h"
-
-/* Define private replacements for stdin, stdout, and stderr. */
-FILE *gs_stdio[3];
-
+#include "stdint_.h"
 
 /* ------ Redirected stdout and stderr  ------ */
 
 #include <stdarg.h>
 #define PRINTF_BUF_LENGTH 1024
 
-int outprintf(const char *fmt, ...)
+int outprintf(const gs_memory_t *mem, const char *fmt, ...)
 {
     int count;
     char buf[PRINTF_BUF_LENGTH];
@@ -69,12 +71,12 @@ int outprintf(const char *fmt, ...)
     va_start(args, fmt);
 
     count = vsprintf(buf, fmt, args);
-    outwrite(buf, count);
+    outwrite(mem, buf, count);
     if (count >= PRINTF_BUF_LENGTH) {
 	count = sprintf(buf, 
 	    "PANIC: printf exceeded %d bytes.  Stack has been corrupted.\n", 
 	    PRINTF_BUF_LENGTH);
-	outwrite(buf, count);
+	outwrite(mem, buf, count);
     }
     va_end(args);
     return count;
@@ -132,13 +134,13 @@ dflush(void)
 {
     errflush();
 }
-private const char *
+static const char *
 dprintf_file_tail(const char *file)
 {
     const char *tail = file + strlen(file);
 
     while (tail > file &&
-	   (isalnum(tail[-1]) || tail[-1] == '.' || tail[-1] == '_')
+	   (isalnum((unsigned char)tail[-1]) || tail[-1] == '.' || tail[-1] == '_')
 	)
 	--tail;
     return tail;
@@ -160,14 +162,14 @@ dprintf_file_only(const char *file)
 }
 #endif
 void
-printf_program_ident(const char *program_name, long revision_number)
+printf_program_ident(const gs_memory_t *mem, const char *program_name, long revision_number)
 {
     if (program_name)
-	outprintf((revision_number ? "%s " : "%s"), program_name);
+        outprintf(mem, (revision_number ? "%s " : "%s"), program_name);
     if (revision_number) {
 	int fpart = revision_number % 100;
 
-	outprintf("%d.%02d", (int)(revision_number / 100), fpart);
+	outprintf(mem, "%d.%02d", (int)(revision_number / 100), fpart);
     }
 }
 void
@@ -216,18 +218,82 @@ gs_log_error(int err, const char *file, int line)
 
 /* Check for interrupts before a return. */
 int
-gs_return_check_interrupt(int code)
+gs_return_check_interrupt(const gs_memory_t *mem, int code)
 {
     if (code < 0)
 	return code;
     {
-	int icode = gp_check_interrupts();
+	int icode = gp_check_interrupts(mem);
 
 	return (icode == 0 ? code :
 		gs_note_error((icode > 0 ? gs_error_interrupt : icode)));
     }
 }
 
+
+int gs_throw_imp(const char *func, const char *file, int line, int op, int code, const char *fmt, ...)
+{
+    char msg[1024];
+    va_list ap;
+
+    va_start(ap, fmt);
+    vsprintf(msg, fmt, ap);
+    msg[sizeof(msg) - 1] = 0;
+    va_end(ap);
+
+    if (!gs_debug_c('#')) {
+	; /* NB: gs_log_errors  
+	   * we could disable these printfs, and probably will when, 
+	   * the code becomes more stable: 
+	   * return code;
+	   */
+    }
+       
+
+    /* throw */
+    if (op == 0)
+	errprintf("+ %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* rethrow */
+    if (op == 1)
+	errprintf("| %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* catch */
+    if (op == 2)
+	errprintf("- %s:%d: %s(): %s\n", file, line, func, msg);
+
+    /* warn */
+    if (op == 3)
+	errprintf("  %s:%d: %s(): %s\n", file, line, func, msg);
+
+    return code;
+}
+
+const char *gs_errstr(int code)
+{
+    switch (code) {
+    default:
+    case gs_error_unknownerror: return "unknownerror";
+    case gs_error_interrupt: return "interrupt";
+    case gs_error_invalidaccess: return "invalidaccess";
+    case gs_error_invalidfileaccess: return "invalidfileaccess";
+    case gs_error_invalidfont: return "invalidfont";
+    case gs_error_ioerror: return "ioerror";
+    case gs_error_limitcheck: return "limitcheck";
+    case gs_error_nocurrentpoint: return "nocurrentpoint";
+    case gs_error_rangecheck: return "rangecheck";
+    case gs_error_typecheck: return "typecheck";
+    case gs_error_undefined: return "undefined";
+    case gs_error_undefinedfilename: return "undefinedfilename";
+    case gs_error_undefinedresult: return "undefinedresult";
+    case gs_error_VMerror: return "vmerror";
+    case gs_error_unregistered: return "unregistered";
+    case gs_error_hit_detected: return "hit_detected";
+    case gs_error_Fatal: return "Fatal";
+    }
+}
+
+
 /* ------ Substitutes for missing C library functions ------ */
 
 #ifdef MEMORY__NEED_MEMMOVE	/* see memory_.h */
@@ -537,7 +603,7 @@ ilog2(int n)
     while (m >= 16)
 	m >>= 4, l += 4;
     return
-	(m <= 1 ? 0 :
+	(m <= 1 ? l :
 	 "\000\000\001\001\002\002\002\002\003\003\003\003\003\003\003\003"[m] + l);
 }
 
@@ -1046,7 +1112,7 @@ gs_sqrt(double x, const char *file, int line)
 #define sin89 0.99984769515639127
 #define sin90 1.00000000000000000
 
-private const double sin_table[361] =
+static const double sin_table[361] =
 {
     sin0,
     sin1, sin2, sin3, sin4, sin5, sin6, sin7, sin8, sin9, sin10,
@@ -1143,10 +1209,16 @@ gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 static const int isincos[5] =
 {0, 1, 0, -1, 0};
 
+/* GCC with -ffast-math compiles ang/90. as ang*(1/90.), losing precission.
+ * This doesn't happen when the numeral is replaced with a non-const variable.
+ * So we define the variable to work around the GCC problem. 
+ */
+static double const_90_degrees = 90.;
+
 double
 gs_sin_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/*
@@ -1161,7 +1233,7 @@ gs_sin_degrees(double ang)
 double
 gs_cos_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
@@ -1173,7 +1245,7 @@ gs_cos_degrees(double ang)
 void
 gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
@@ -1214,3 +1286,116 @@ gs_atan2_degrees(double y, double x, double *pangle)
     }
     return 0;
 }
+
+/*
+ * Define a function for finding intersection of small bars.
+ * Coordinates must be so small that their cubes fit into 60 bits.
+ * This function doesn't check intersections at end of bars,
+ * so  the caller must care of them on necessity.
+ * Returns : *ry is the Y-coordinate of the intersection
+ * truncated to 'fixed'; *ey is 1 iff the precise Y coordinate of
+ * the intersection is greater than *ry (used by the shading algorithm).
+ */
+bool 
+gx_intersect_small_bars(fixed q0x, fixed q0y, fixed q1x, fixed q1y, fixed q2x, fixed q2y, 
+			fixed q3x, fixed q3y, fixed *ry, fixed *ey)
+{
+    fixed dx1 = q1x - q0x, dy1 = q1y - q0y;
+    fixed dx2 = q2x - q0x, dy2 = q2y - q0y;
+    fixed dx3 = q3x - q0x, dy3 = q3y - q0y;
+
+    int64_t vp2a, vp2b, vp3a, vp3b;
+    int s2, s3;
+
+    if (dx1 == 0 && dy1 == 0)
+	return false; /* Zero length bars are out of interest. */
+    if (dx2 == 0 && dy2 == 0)
+	return false; /* Contacting ends are out of interest. */
+    if (dx3 == 0 && dy3 == 0)
+	return false; /* Contacting ends are out of interest. */
+    if (dx2 == dx1 && dy2 == dy1)
+	return false; /* Contacting ends are out of interest. */
+    if (dx3 == dx1 && dy3 == dy1)
+	return false; /* Contacting ends are out of interest. */
+    if (dx2 == dx3 && dy2 == dy3)
+	return false; /* Zero length bars are out of interest. */
+    vp2a = (int64_t)dx1 * dy2;
+    vp2b = (int64_t)dy1 * dx2; 
+    /* vp2 = vp2a - vp2b; It can overflow int64_t, but we only need the sign. */
+    if (vp2a > vp2b)
+	s2 = 1;
+    else if (vp2a < vp2b)
+	s2 = -1;
+    else 
+	s2 = 0;
+    vp3a = (int64_t)dx1 * dy3;
+    vp3b = (int64_t)dy1 * dx3; 
+    /* vp3 = vp3a - vp3b; It can overflow int64_t, but we only need the sign. */
+    if (vp3a > vp3b)
+	s3 = 1;
+    else if (vp3a < vp3b)
+	s3 = -1;
+    else 
+	s3 = 0;
+    if (s2 == 0) {
+	if (s3 == 0)
+	    return false; /* Collinear bars - out of interest. */
+	if (0 <= dx2 && dx2 <= dx1 && 0 <= dy2 && dy2 <= dy1) {
+	    /* The start of the bar 2 is in the bar 1. */
+	    *ry = q2y;
+	    *ey = 0;
+	    return true;
+	}
+    } else if (s3 == 0) {
+	if (0 <= dx3 && dx3 <= dx1 && 0 <= dy3 && dy3 <= dy1) {
+	    /* The end of the bar 2 is in the bar 1. */
+	    *ry = q3y;
+	    *ey = 0;
+	    return true;
+	}
+    } else if (s2 * s3 < 0) {
+	/* The intersection definitely exists, so the determinant isn't zero.  */
+	fixed d23x = dx3 - dx2, d23y = dy3 - dy2;
+	int64_t det = (int64_t)dx1 * d23y - (int64_t)dy1 * d23x;
+	int64_t mul = (int64_t)dx2 * d23y - (int64_t)dy2 * d23x;
+	{
+	    /* Assuming small bars : cubes of coordinates must fit into int64_t.
+	       curve_samples must provide that.  */
+	    int64_t num = dy1 * mul, iiy;
+	    fixed iy;
+	    fixed pry, pey;
+
+	    {	/* Likely when called form wedge_trap_decompose or constant_color_quadrangle,
+		   we always have det > 0 && num >= 0, but we check here for a safety reason. */
+		if (det < 0)
+		    num = -num, det = -det;
+		iiy = (num >= 0 ? num / det : (num - det + 1) / det);
+		iy = (fixed)iiy;
+		if (iy != iiy) {
+		    /* If it is inside the bars, it must fit into fixed. */
+		    return false;
+		}
+	    }
+	    if (dy1 > 0) {
+		if (iy < 0 || iy >= dy1)
+		    return false; /* Outside the bar 1. */
+	    } else {
+		if (iy > 0 || iy <= dy1)
+		    return false; /* Outside the bar 1. */
+	    }
+	    if (dy2 < dy3) {
+		if (iy <= dy2 || iy >= dy3)
+		    return false; /* Outside the bar 2. */
+	    } else {
+		if (iy >= dy2 || iy <= dy3)
+		    return false; /* Outside the bar 2. */
+	    }
+	    pry = q0y + (fixed)iy;
+	    pey = (iy * det < num ? 1 : 0);
+	    *ry = pry;
+	    *ey = pey;
+	}
+	return true;
+    }
+    return false;
+}
