@@ -1,51 +1,42 @@
-/* Copyright (C) 1992, 1993, 1994, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsutil.c,v 1.4.6.2.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gsutil.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Utilities for Ghostscript library */
+
 #include "string_.h"
 #include "memory_.h"
 #include "gstypes.h"
 #include "gconfigv.h"		/* for USE_ASM */
+#include "gserror.h"
+#include "gserrors.h"
 #include "gsmemory.h"		/* for init procedure */
 #include "gsrect.h"		/* for prototypes */
 #include "gsuid.h"
 #include "gsutil.h"		/* for prototypes */
+#include "gzstate.h"
+#include "gxdcolor.h"
+
+
 
 /* ------ Unique IDs ------ */
 
-/* Generate a block of unique IDs. */
-static ulong gs_next_id;
-
-init_proc(gs_gsutil_init);	/* check prototype */
-int
-gs_gsutil_init(gs_memory_t *mem)
-{
-    gs_next_id = 1;
-    return 0;
-}
-
 ulong
-gs_next_ids(uint count)
+gs_next_ids(const gs_memory_t *mem, uint count)
 {
-    ulong id = gs_next_id;
+    ulong id = mem->gs_lib_ctx->gs_next_id;
 
-    gs_next_id += count;
+    mem->gs_lib_ctx->gs_next_id += count;
     return id;
 }
 
@@ -75,19 +66,19 @@ memflip8x8(const byte * inp, int line_size, byte * outp, int dist)
     /* Check for all 8 bytes being the same. */
     /* This is especially worth doing for the case where all are zero. */
     if (aceg == bdfh && (aceg >> 8) == (aceg & 0xffffff)) {
-	if (aceg == 0)
+	if (aceg == 0 || aceg == 0xffffffff)
 	    goto store;
-	*outp = -((aceg >> 7) & 1);
-	outp[dist] = -((aceg >> 6) & 1);
+	*outp = (byte)-(int)((aceg >> 7) & 1);
+	outp[dist] = (byte)-(int)((aceg >> 6) & 1);
 	outp += dist << 1;
-	*outp = -((aceg >> 5) & 1);
-	outp[dist] = -((aceg >> 4) & 1);
+	*outp = (byte)-(int)((aceg >> 5) & 1);
+	outp[dist] = (byte)-(int)((aceg >> 4) & 1);
 	outp += dist << 1;
-	*outp = -((aceg >> 3) & 1);
-	outp[dist] = -((aceg >> 2) & 1);
+	*outp = (byte)-(int)((aceg >> 3) & 1);
+	outp[dist] = (byte)-(int)((aceg >> 2) & 1);
 	outp += dist << 1;
-	*outp = -((aceg >> 1) & 1);
-	outp[dist] = -(aceg & 1);
+	*outp = (byte)-(int)((aceg >> 1) & 1);
+	outp[dist] = (byte)-(int)(aceg & 1);
 	return;
     } {
 	register uint temp;
@@ -237,6 +228,23 @@ uid_equal(register const gs_uid * puid1, register const gs_uid * puid2)
 		(uint) - (puid1->id) * sizeof(long));
 }
 
+/* Copy the XUID data for a uid, if needed, updating the uid in place. */
+int
+uid_copy(gs_uid *puid, gs_memory_t *mem, client_name_t cname)
+{
+    if (uid_is_XUID(puid)) {
+	uint xsize = uid_XUID_size(puid);
+	long *xvalues = (long *)
+	    gs_alloc_byte_array(mem, xsize, sizeof(long), cname);
+
+	if (xvalues == 0)
+	    return_error(gs_error_VMerror);
+	memcpy(xvalues, uid_XUID_values(puid), xsize * sizeof(long));
+	puid->xvalues = xvalues;
+    }
+    return 0;
+}
+
 /* ------ Rectangle utilities ------ */
 
 /*
@@ -278,3 +286,38 @@ int_rect_difference(gs_int_rect * outer, const gs_int_rect * inner,
     }
     return count;
 }
+
+/* tag stuff */
+static gs_object_tag_type_t BITTAG = GS_DEVICE_DOESNT_SUPPORT_TAGS;
+
+void
+gs_enable_object_tagging()
+{
+    if (BITTAG == GS_DEVICE_DOESNT_SUPPORT_TAGS)
+        BITTAG = GS_UNKNOWN_TAG;
+}
+
+void
+gs_set_object_tag(gs_state * pgs, const gs_object_tag_type_t tag)
+{
+    if (BITTAG != GS_DEVICE_DOESNT_SUPPORT_TAGS) {
+		if ( pgs->object_tag != tag ) {
+			pgs->object_tag = tag;
+			/* mkromfs breaks this dependance 
+			   NB: needs to be fixed.
+			gx_unset_dev_color(pgs);
+			**/
+			BITTAG = tag;
+			/* the assumption is made that the caller will:
+			 * gx_set_dev_color(pgs);
+			 */
+		}
+    }
+}
+
+gs_object_tag_type_t
+gs_current_object_tag()
+{
+    return BITTAG;
+}
+
