@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxalloc.h 6707 2006-04-16 19:01:41Z leonardo $ */
+/* $Id: gxalloc.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Structure definitions for standard allocator */
 /* Requires gsmemory.h, gsstruct.h */
 
@@ -26,8 +26,6 @@ typedef struct gs_ref_memory_s gs_ref_memory_t;
 #include "gsalloc.h"
 #include "gxobj.h"
 
-#define NO_INVISIBLE_LEVELS 1 /* old code = 0, new code = 1 */
-
 /* ================ Chunks ================ */
 
 /*
@@ -382,9 +380,7 @@ struct gs_ref_memory_s {
     /* Sharing / saved state information */
     int num_contexts;		/* # of contexts sharing this VM */
     struct alloc_change_s *changes;
-#if NO_INVISIBLE_LEVELS
     struct alloc_change_s *scan_limit;
-#endif
     struct alloc_save_s *saved;
     long total_scanned;
     struct alloc_save_s *reloc_saved;	/* for GC */
