@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 1998 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxbitops.h,v 1.2.6.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gxbitops.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Internal definitions for bitmap operations */
 
 #ifndef gxbitops_INCLUDED
@@ -50,8 +45,8 @@
 #  define chunk_bit_mask cbit_mask(chunk)
 #define calign_bytes(ct)\
   (sizeof(ct) == 1 ? 1:\
-   sizeof(ct) == sizeof(short) ? arch_align_short_mod :\
-   sizeof(ct) == sizeof(int) ? arch_align_int_mod : arch_align_long_mod)
+   sizeof(ct) == sizeof(short) ? ARCH_ALIGN_SHORT_MOD :\
+   sizeof(ct) == sizeof(int) ? ARCH_ALIGN_INT_MOD : ARCH_ALIGN_LONG_MOD)
 #  define chunk_align_bytes calign_bytes(chunk)
 #define calign_bit_mask(ct) (calign_bytes(ct)*8-1)
 #  define chunk_align_bit_mask calign_bit_mask(chunk)
@@ -81,7 +76,7 @@
 /* Define whether this is a machine where chunks are long, */
 /* but the machine can't shift a long by its full width. */
 #define arch_cant_shift_full_chunk\
-  (arch_is_big_endian && !arch_ints_are_short && !arch_can_shift_full_long)
+  (ARCH_IS_BIG_ENDIAN && !ARCH_INTS_ARE_SHORT && !ARCH_CAN_SHIFT_FULL_LONG)
 
 /* Pointer arithmetic macros. */
 #define inc_ptr(ptr,delta)\
