@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclmem.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxclmem.c 8250 2007-09-25 13:31:24Z giles $ */
 /* RAM-based command list implementation */
 #include "memory_.h"
 #include "gx.h"
@@ -131,7 +131,7 @@ LIMITATIONS.
    least as large as the fixed overhead of the compressor plus the
    decompressor, plus the expected compressed size of a block that size.
  */
-private const long COMPRESSION_THRESHOLD = 300000;
+static const long COMPRESSION_THRESHOLD = 32000000;
 
 #define NEED_TO_COMPRESS(f)\
   ((f)->ok_to_compress && (f)->total_space > COMPRESSION_THRESHOLD)
@@ -150,8 +150,10 @@ private const long COMPRESSION_THRESHOLD = 300000;
 private_st_MEMFILE();
 
 	/* forward references */
-private void memfile_free_mem(MEMFILE * f);
-private int memfile_init_empty(MEMFILE * f);
+static void memfile_free_mem(MEMFILE * f);
+static int memfile_init_empty(MEMFILE * f);
+static int memfile_set_memory_warning(clist_file_ptr cf, int bytes_left);
+static int memfile_fclose(clist_file_ptr cf, const char *fname, bool delete);
 
 /************************************************/
 /*   #define DEBUG      /- force statistics -/  */
@@ -176,7 +178,7 @@ const byte *decomp_rd_ptr1, *decomp_rd_limit1;
 #endif
 
 /* ----------------------------- Memory Allocation --------------------- */
-void *	/* allocated memory's address, 0 if failure */
+static void *	/* allocated memory's address, 0 if failure */
 allocateWithReserve(
          MEMFILE  *f,			/* file to allocate mem to */
          int      sizeofBlock,		/* size of block to allocate */
@@ -218,7 +220,7 @@ allocateWithReserve(
 
 /* ---------------- Open/close/unlink ---------------- */
 
-int
+static int
 memfile_fopen(char fname[gp_file_name_sizeof], const char *fmode,
 	      clist_file_ptr /*MEMFILE * */  * pf,
 	      gs_memory_t *mem, gs_memory_t *data_mem, bool ok_to_compress)
@@ -319,7 +321,7 @@ finish:
     return code;
 }
 
-int
+static int
 memfile_fclose(clist_file_ptr cf, const char *fname, bool delete)
 {
     MEMFILE *const f = (MEMFILE *)cf;
@@ -355,7 +357,7 @@ memfile_fclose(clist_file_ptr cf, const char *fname, bool delete)
     return 0;
 }
 
-int
+static int
 memfile_unlink(const char *fname)
 {
     /*
@@ -368,7 +370,7 @@ memfile_unlink(const char *fname)
 /* ---------------- Writing ---------------- */
 
 /* Pre-alloc enough reserve mem blox to guarantee a write of N bytes will succeed */
-int	/* returns 0 ok, gs_error_VMerror if insufficient */
+static int	/* returns 0 ok, gs_error_VMerror if insufficient */
 memfile_set_memory_warning(clist_file_ptr cf, int bytes_left)
 {
     MEMFILE *const f = (MEMFILE *)cf;
@@ -433,7 +435,7 @@ finish:
     return code;
 }
 
-private int
+static int
 compress_log_blk(MEMFILE * f, LOG_MEMFILE_BLK * bp)
 {
     int status;
@@ -500,7 +502,7 @@ compress_log_blk(MEMFILE * f, LOG_MEMFILE_BLK * bp)
 }				/* end "compress_log_blk()"                                     */
 
 /*      Internal (private) routine to handle end of logical block       */
-private int	/* ret 0 ok, -ve error, or +ve low-memory warning */
+static int	/* ret 0 ok, -ve error, or +ve low-memory warning */
 memfile_next_blk(MEMFILE * f)
 {
     LOG_MEMFILE_BLK *bp = f->log_curr_blk;
@@ -611,7 +613,7 @@ memfile_next_blk(MEMFILE * f)
     return (ecode);
 }
 
-int	/* returns # of chars actually written */
+static int	/* returns # of chars actually written */
 memfile_fwrite_chars(const void *data, uint len, clist_file_ptr cf)
 {
     const char *str = (const char *)data;
@@ -667,7 +669,7 @@ memfile_fwrite_chars(const void *data, uint len, clist_file_ptr cf)
 /*      and decompress it.                                              */
 /*                                                                      */
 
-private int
+static int
 memfile_get_pdata(MEMFILE * f)
 {
     int i, num_raw_buffers, status;
@@ -824,7 +826,7 @@ memfile_get_pdata(MEMFILE * f)
 
 /* ---------------- Reading ---------------- */
 
-int
+static int
 memfile_fread_chars(void *data, uint len, clist_file_ptr cf)
 {
     char *str = (char *)data;
@@ -857,19 +859,19 @@ memfile_fread_chars(void *data, uint len, clist_file_ptr cf)
 
 /* ---------------- Position/status ---------------- */
 
-int
+static int
 memfile_ferror_code(clist_file_ptr cf)
 {
     return (((MEMFILE *) cf)->error_code);	/* errors stored here */
 }
 
-long
+static int64_t
 memfile_ftell(clist_file_ptr cf)
 {
     return (((MEMFILE *) cf)->log_curr_pos);
 }
 
-void
+static void
 memfile_rewind(clist_file_ptr cf, bool discard_data, const char *ignore_fname)
 {
     MEMFILE *f = (MEMFILE *) cf;
@@ -885,8 +887,8 @@ memfile_rewind(clist_file_ptr cf, bool discard_data, const char *ignore_fname)
     }
 }
 
-int
-memfile_fseek(clist_file_ptr cf, long offset, int mode, const char *ignore_fname)
+static int
+memfile_fseek(clist_file_ptr cf, int64_t offset, int mode, const char *ignore_fname)
 {
     MEMFILE *f = (MEMFILE *) cf;
     long i, block_num, new_pos;
@@ -931,7 +933,7 @@ memfile_fseek(clist_file_ptr cf, long offset, int mode, const char *ignore_fname
 
 /* ---------------- Internal routines ---------------- */
 
-private void
+static void
 memfile_free_mem(MEMFILE * f)
 {
     LOG_MEMFILE_BLK *bp, *tmpbp;
@@ -1076,7 +1078,7 @@ memfile_free_mem(MEMFILE * f)
     }
 }
 
-private int
+static int
 memfile_init_empty(MEMFILE * f)
 {
     PHYS_MEMFILE_BLK *pphys;
@@ -1122,3 +1124,24 @@ memfile_init_empty(MEMFILE * f)
 
     return 0;
 }
+
+clist_io_procs_t clist_io_procs_memory = {
+    memfile_fopen,
+    memfile_fclose,
+    memfile_unlink,
+    memfile_fwrite_chars,
+    memfile_fread_chars,
+    memfile_set_memory_warning,
+    memfile_ferror_code,
+    memfile_ftell,
+    memfile_rewind,
+    memfile_fseek,
+};
+
+init_proc(gs_gxclmem_init);
+int
+gs_gxclmem_init(gs_memory_t *mem)
+{
+    clist_io_procs_memory_global = &clist_io_procs_memory;
+    return 0;
+}
