@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclpath.c 8134 2007-07-17 09:55:56Z leonardo $ */
+/*$Id: gxclpath.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Higher-level path operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -37,7 +37,7 @@ ulong stats_cmd_diffs[5];
 #endif
 
 /* Forward declarations */
-private int cmd_put_path(gx_device_clist_writer * cldev,
+static int cmd_put_path(gx_device_clist_writer * cldev,
 			 gx_clist_state * pcls, const gx_path * ppath,
 			 fixed ymin, fixed ymax, byte op,
 			 bool implicit_close, segment_notes keep_notes);
@@ -45,7 +45,7 @@ private int cmd_put_path(gx_device_clist_writer * cldev,
 /* ------ Utilities ------ */
 
 /* Compute the colors used by a colored halftone. */
-private gx_color_index
+static gx_color_index
 colored_halftone_colors_used(gx_device_clist_writer *cldev,
 			     const gx_drawing_color *pdcolor)
 {
@@ -88,7 +88,7 @@ cmd_slow_rop(gx_device *dev, gs_logical_operation_t lop,
 	    rop = rop3_know_T_1(rop);
     }
     return !(rop == rop3_0 || rop == rop3_1 ||
-	     rop == rop3_D || rop == rop3_S || rop == rop3_T);
+	     rop == rop3_S || rop == rop3_T);
 }
 
 /* Write out the color for filling, stroking, or masking. */
@@ -114,16 +114,6 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         color_unset(psdc);
     }
 
-    /* see if phase informaiton must be inserted in the command list */
-    if ( pdcolor->type->get_phase(pdcolor, &color_phase) &&
-         (color_phase.x != pcls->tile_phase.x ||
-          color_phase.y != pcls->tile_phase.y   )        &&
-	 (code = cmd_set_tile_phase( cldev,
-                                     pcls,
-                                     color_phase.x,
-                                     color_phase.y )) < 0  )
-        return code;
-
     /*
      * Get the device color type index and the required size.
      *
@@ -147,6 +137,16 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         return code;
     req_size = dc_size + 2 + 1 + enc_u_sizew(dc_size);
 
+    /* see if phase informaiton must be inserted in the command list */
+    if ( pdcolor->type->get_phase(pdcolor, &color_phase) &&
+         (color_phase.x != pcls->tile_phase.x ||
+          color_phase.y != pcls->tile_phase.y   )        &&
+	 (code = cmd_set_tile_phase( cldev,
+                                     pcls,
+                                     color_phase.x,
+                                     color_phase.y )) < 0  )
+        return code;
+
     /*
      * Encoded device colors are small in comparison to the command
      * buffer size (< 64 bytes), so we can just clear space in the
@@ -231,7 +231,7 @@ cmd_check_clip_path(gx_device_clist_writer * cldev, const gx_clip_path * pcpath)
 #define FILL_KNOWN\
  (cj_ac_sa_known | flatness_known | op_bm_tk_known | opacity_alpha_known |\
   shape_alpha_known | fill_adjust_known | alpha_known | clip_path_known)
-private void
+static void
 cmd_check_fill_known(gx_device_clist_writer *cdev, const gs_imager_state *pis,
 		     floatp flatness, const gs_fixed_point *padjust,
 		     const gx_clip_path *pcpath, uint *punknown)
@@ -832,7 +832,7 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
  * sometimes used for images, so its performance does matter.
  */
 
-private int
+static int
 clist_put_polyfill(gx_device *dev, fixed px, fixed py,
 		   const gs_fixed_point *points, int num_points,
 		   const gx_drawing_color *pdcolor, gs_logical_operation_t lop)
@@ -945,7 +945,7 @@ typedef struct cmd_segment_writer_s {
 cmd_segment_writer;
 
 /* Put out a path segment command. */
-private int
+static int
 cmd_put_segment(cmd_segment_writer * psw, byte op,
 		const fixed * operands, segment_notes notes)
 {
@@ -1081,7 +1081,7 @@ cmd_put_segment(cmd_segment_writer * psw, byte op,
  * Write a path.  We go to a lot of trouble to omit segments that are
  * entirely outside the band.
  */
-private int
+static int
 cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	     const gx_path * ppath, fixed ymin, fixed ymax, byte path_op,
 	     bool implicit_close, segment_notes keep_notes)
