@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclrect.c 8134 2007-07-17 09:55:56Z leonardo $ */
+/* $Id: gxclrect.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Rectangle-oriented command writing for command list */
 #include "gx.h"
 #include "gserrors.h"
@@ -29,14 +29,14 @@
    (rect).width = width, (rect).height = height)
 
 /* Write a rectangle. */
-private int
+static int
 cmd_size_rect(register const gx_cmd_rect * prect)
 {
     return
 	cmd_sizew(prect->x) + cmd_sizew(prect->y) +
 	cmd_sizew(prect->width) + cmd_sizew(prect->height);
 }
-private byte *
+static byte *
 cmd_put_rect(register const gx_cmd_rect * prect, register byte * dp)
 {
     cmd_putw(prect->x, dp);
@@ -134,7 +134,7 @@ cmd_write_rect_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     return 0;
 }
 
-private inline byte * 
+static inline byte * 
 cmd_put_frac31_color(gx_device_clist_writer * cldev, const frac31 *c, byte *dp)
 {
     int num_components = cldev->color_info.num_components;
@@ -145,7 +145,7 @@ cmd_put_frac31_color(gx_device_clist_writer * cldev, const frac31 *c, byte *dp)
     return dp;
 }
 
-private inline int
+static inline int
 cmd_size_frac31_color(gx_device_clist_writer * cldev, const frac31 *c)
 {
     int j, s = 0;
@@ -156,7 +156,7 @@ cmd_size_frac31_color(gx_device_clist_writer * cldev, const frac31 *c)
     return s;
 }
 
-private int
+static int
 cmd_write_trapezoid_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 		    int op,  const gs_fixed_edge *left, const gs_fixed_edge *right,
 		    fixed ybot, fixed ytop, int options, 
@@ -273,7 +273,7 @@ error_in_rect:
     return 0;
 }
 
-private inline int 
+static inline int 
 clist_write_fill_trapezoid(gx_device * dev,
     const gs_fixed_edge *left, const gs_fixed_edge *right,
     fixed ybot, fixed ytop, int options,
