@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclutil.c 8127 2007-07-15 17:27:07Z leonardo $ */
+/* $Id: gxclutil.c 8369 2007-11-15 07:22:43Z leonardo $ */
 /* Command list writing utilities. */
 
 #include "memory_.h"
@@ -31,13 +31,13 @@
 #ifdef DEBUG
 const char *const cmd_op_names[16] =
 {cmd_op_name_strings};
-private const char *const cmd_misc_op_names[16] =
+static const char *const cmd_misc_op_names[16] =
 {cmd_misc_op_name_strings};
-private const char *const cmd_misc2_op_names[16] =
+static const char *const cmd_misc2_op_names[16] =
 {cmd_misc2_op_name_strings};
-private const char *const cmd_segment_op_names[16] =
+static const char *const cmd_segment_op_names[16] =
 {cmd_segment_op_name_strings};
-private const char *const cmd_path_op_names[16] =
+static const char *const cmd_path_op_names[16] =
 {cmd_path_op_name_strings};
 const char *const *const cmd_sub_op_names[16] =
 {cmd_misc_op_names, 0, 0, 0, 0, 0, 0, 0,
@@ -125,7 +125,7 @@ cmd_print_stats(void)
 /* ---------------- Writing utilities ---------------- */
 
 /* Write the commands for one band or band range. */
-private int	/* ret 0 all ok, -ve error code, or +1 ok w/low-mem warning */
+static int	/* ret 0 all ok, -ve error code, or +1 ok w/low-mem warning */
 cmd_write_band(gx_device_clist_writer * cldev, int band_min, int band_max,
 	       cmd_list * pcl, gx_band_complexity_t *band_complexity, byte cmd_end)
 {
@@ -160,6 +160,8 @@ cmd_write_band(gx_device_clist_writer * cldev, int band_min, int band_max,
 		    return_error(gs_error_Fatal);
 		}
 #endif
+		if_debug2('L',"[L]Wrote cmd id=%ld at %ld\n", cp->id, 
+		    (long)cldev->page_info.io_procs->ftell(cfile));
 		cldev->page_info.io_procs->fwrite_chars(cp + 1, cp->size, cfile);
 	    }
 	    pcl->head = pcl->tail = 0;
@@ -244,6 +246,7 @@ cmd_put_list_op(gx_device_clist_writer * cldev, cmd_list * pcl, uint size)
 	    lprintf1("cmd_put_list_op error at 0x%lx\n", (ulong) pcl->tail);
 	}
 #endif
+	if_debug2('L', ", to id=%ld , offset=%ld", pcl->tail->id, pcl->tail->size);
 	pcl->tail->size += size;
     } else {
 	/* Skip to an appropriate alignment boundary. */
@@ -268,6 +271,9 @@ cmd_put_list_op(gx_device_clist_writer * cldev, cmd_list * pcl, uint size)
 	pcl->tail = cp;
 	cldev->ccl = pcl;
 	cp->size = size;
+	cp->id = cldev->ins_count;
+	if_debug1('L', ", id=%ld ", cldev->ins_count);
+	cldev->ins_count++;
     }
     cldev->cnext = dp + size;
     return dp;
@@ -389,7 +395,7 @@ const gx_color_index cmd_delta_offsets[] = {
 	tab_entry(0x0808080808080808),
 	};
 
-private const gx_color_index cmd_delta_masks[] = {
+static const gx_color_index cmd_delta_masks[] = {
 	tab_entry(0),
 	tab_entry(0),
 	tab_entry(0x0f0f),
@@ -694,7 +700,7 @@ cmd_put_params(gx_device_clist_writer *cldev,
 }
 
 /* Initialize CCITTFax filters. */
-private void
+static void
 clist_cf_init(stream_CF_state *ss, int width)
 {
     ss->K = -1;
