@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdevmem.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxdevmem.h 8056 2007-06-18 06:37:58Z ray $ */
 /* Structure and procedures for memory devices */
 /* Requires gxdevice.h */
 
@@ -177,17 +177,17 @@ extern_st(st_device_memory);
  * size includes both the bitmap and the line pointers.
  */
 /* bits only */
-ulong gdev_mem_bits_size(const gx_device_memory *mdev, int width,
-			 int height);
+int gdev_mem_bits_size(const gx_device_memory *mdev, int width,
+			 int height, ulong *size);
 /* line pointers only */
 ulong gdev_mem_line_ptrs_size(const gx_device_memory *mdev, int width,
 			      int height);
 /* bits + line pointers */
-ulong gdev_mem_data_size(const gx_device_memory *mdev, int width,
-			 int height);
+int gdev_mem_data_size(const gx_device_memory *mdev, int width,
+			 int height, ulong *size);
 
-#define gdev_mem_bitmap_size(mdev)\
-  gdev_mem_data_size(mdev, (mdev)->width, (mdev)->height)
+#define gdev_mem_bitmap_size(mdev, size)\
+  gdev_mem_data_size(mdev, (mdev)->width, (mdev)->height, size)
 
 /*
  * Do the inverse computation: given the device width and a buffer size,
