@@ -1,22 +1,17 @@
-/* Copyright (C) 1995, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxidata.c,v 1.4.6.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gxidata.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Generic image enumeration and cleanup */
 #include "gx.h"
 #include "memory_.h"
@@ -26,12 +21,12 @@
 #include "gximage.h"
 
 /* Forward declarations */
-private void update_strip(P1(gx_image_enum *penum));
-private void repack_bit_planes(P7(const gx_image_plane_t *src_planes,
-				  const ulong *offsets, int num_planes,
-				  byte *buffer, int width,
-				  const sample_lookup_t * ptab, int spread));
-private gx_device *setup_image_device(P1(const gx_image_enum *penum));
+static void update_strip(gx_image_enum *penum);
+static void repack_bit_planes(const gx_image_plane_t *src_planes,
+			       const ulong *offsets, int num_planes,
+			       byte *buffer, int width,
+			       const sample_lookup_t * ptab, int spread);
+static gx_device *setup_image_device(const gx_image_enum *penum);
 
 /* Process the next piece of an ImageType 1 image. */
 int
@@ -45,6 +40,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
     int y_end = min(y + height, penum->rect.h);
     int width_spp = penum->rect.w * penum->spp;
     int num_planes = penum->num_planes;
+    int num_components_per_plane = 1;
 
 #define BCOUNT(plane)		/* bytes per data row */\
   (((penum->rect.w + (plane).data_x) * penum->spp * penum->bps / num_planes\
@@ -76,6 +72,10 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
 	penum->used.y = 0;
     } else
 	memset(offsets, 0, num_planes * sizeof(offsets[0]));
+    if (num_planes == 1 && penum->plane_depths[0] != penum->bps) {
+	/* A single plane with multiple components. */
+	num_components_per_plane = penum->plane_depths[0] / penum->bps;
+    }
     for (; penum->y < y_end; penum->y++) {
 	int px;
 	const byte *buffer;
@@ -89,7 +89,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
 	    sourcex = 0;
 	    for (px = 0; px < num_planes; px += penum->bps)
 		repack_bit_planes(planes, offsets, penum->bps, penum->buffer,
-				  penum->rect.w, &penum->map[0].table,
+				  penum->rect.w, &penum->map[px].table,
 				  penum->spread);
 	    for (px = 0; px < num_planes; ++px)
 		offsets[px] += planes[px].raster;
@@ -104,7 +104,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
 		(*penum->unpack)(penum->buffer, &sourcex,
 				 planes[0].data + offsets[0],
 				 planes[0].data_x, BCOUNT(planes[0]),
-				 &penum->map[0].table, penum->spread);
+				 &penum->map[0], penum->spread, num_components_per_plane);
 
 	    offsets[0] += planes[0].raster;
 	    for (px = 1; px < num_planes; ++px) {
@@ -112,7 +112,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
 				 &ignore_data_x,
 				 planes[px].data + offsets[px],
 				 planes[px].data_x, BCOUNT(planes[px]),
-				 &penum->map[px].table, penum->spread);
+				 &penum->map[px], penum->spread, 1);
 		offsets[px] += planes[px].raster;
 	    }
 	}
@@ -275,7 +275,7 @@ gx_image1_flush(gx_image_enum_common_t * info)
 }
 
 /* Update the strip DDA when moving to a new row. */
-private void
+static void
 update_strip(gx_image_enum *penum)
 {
     dda_translate(penum->dda.strip.x, penum->cur.x - penum->prev.x);
@@ -289,7 +289,7 @@ update_strip(gx_image_enum *penum)
  * This procedure repacks one row, so the only relevant members of
  * src_planes are data and data_x (not raster).
  */
-private void
+static void
 repack_bit_planes(const gx_image_plane_t *src_planes, const ulong *offsets,
 		  int num_planes, byte *buffer, int width,
 		  const sample_lookup_t * ptab, int spread)
@@ -391,7 +391,7 @@ repack_bit_planes(const gx_image_plane_t *src_planes, const ulong *offsets,
 }
 
 /* Set up the device for drawing an image. */
-private gx_device *
+static gx_device *
 setup_image_device(const gx_image_enum *penum)
 {
     gx_device *dev = penum->dev;
@@ -428,8 +428,17 @@ gx_image1_end_image(gx_image_enum_common_t * info, bool draw_last)
 	if (code < 0)
 	    return code;
     }
+
+   /* release the reference to the target */
+    if ( penum->rop_dev )
+        gx_device_set_target((gx_device_forward *)penum->rop_dev, NULL);
+    if ( penum->clip_dev )
+        gx_device_set_target((gx_device_forward *)penum->clip_dev, NULL);
+    /* it is not clear (to me) why these are freed explicitly instead
+       of using reference counting */
     gs_free_object(mem, penum->rop_dev, "image RasterOp");
     gs_free_object(mem, penum->clip_dev, "image clipper");
+
     if (scaler != 0) {
 	(*scaler->template->release) ((stream_state *) scaler);
 	gs_free_object(mem, scaler, "image scaler state");
