@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gximage.h,v 1.2.6.1.2.1 2003/01/17 00:49:04 giles Exp $ */
+/* $Id: gximage.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Default image rendering state structure */
 /* Requires gxcpath.h, gxdevmem.h, gxdcolor.h, gzpath.h */
 
@@ -55,7 +50,7 @@ typedef enum {
     sd_lookup,			/* use lookup_decode table */
     sd_compute			/* compute using base and factor */
 } sample_decoding;
-typedef struct sample_map_s {
+struct sample_map_s {
 
     sample_lookup_t table;
 
@@ -91,7 +86,12 @@ typedef struct sample_map_s {
 
     bool inverted;
 
-} sample_map;
+};
+
+#ifndef sample_map_DEFINED
+#define sample_map_DEFINED
+typedef struct sample_map_s sample_map;
+#endif
 
 /* Decode an 8-bit sample into a floating point color component. */
 /* penum points to the gx_image_enum structure. */
@@ -121,6 +121,12 @@ typedef struct sample_map_s {
  */
 extern const sample_unpack_proc_t sample_unpack_12_proc;
 
+/*
+ * Declare the pointer that holds the 16-bit unpacking procedure
+ * if 16-bit samples are supported, 0 otherwise.
+ */
+extern const sample_unpack_proc_t sample_unpack_16_proc;
+
 /* Define the distinct postures of an image. */
 /* Each posture includes its reflected variant. */
 typedef enum {
@@ -273,8 +279,8 @@ struct gx_image_enum_s {
  * a full byte, and complement and swap them if the map incorporates
  * a Decode = [1 0] inversion.
  */
-void gx_image_scale_mask_colors(P2(gx_image_enum *penum,
-				   int component_index));
+void gx_image_scale_mask_colors(gx_image_enum *penum,
+				int component_index);
 
 /*
  * Do common initialization for processing an ImageType 1 or 4 image.
@@ -282,9 +288,9 @@ void gx_image_scale_mask_colors(P2(gx_image_enum *penum,
  *	rect
  */
 int
-gx_image_enum_alloc(P4(const gs_image_common_t * pic,
-		       const gs_int_rect * prect,
-		       gs_memory_t * mem, gx_image_enum **ppenum));
+gx_image_enum_alloc(const gs_image_common_t * pic,
+		    const gs_int_rect * prect,
+		    gs_memory_t * mem, gx_image_enum **ppenum);
 
 /*
  * Finish initialization for processing an ImageType 1 or 4 image.
@@ -294,10 +300,17 @@ gx_image_enum_alloc(P4(const gs_image_common_t * pic,
  *	masked, adjust
  */
 int
-gx_image_enum_begin(P8(gx_device * dev, const gs_imager_state * pis,
-		       const gs_matrix *pmat, const gs_image_common_t * pic,
-		       const gx_drawing_color * pdcolor,
-		       const gx_clip_path * pcpath,
-		       gs_memory_t * mem, gx_image_enum *penum));
+gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
+		    const gs_matrix *pmat, const gs_image_common_t * pic,
+		    const gx_drawing_color * pdcolor,
+		    const gx_clip_path * pcpath,
+		    gs_memory_t * mem, gx_image_enum *penum);
+
+/*
+ * Clear the relevant clues. Exported for use by image_render_*
+ * when ht_tile cache is invalidated.
+ */
+void
+image_init_clues(gx_image_enum * penum, int bps, int spp);
 
 #endif /* gximage_INCLUDED */
