@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxiscale.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: gxiscale.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Interpolated image procedures */
 #include "gx.h"
 #include "math_.h"
@@ -48,7 +48,7 @@
 iclass_proc(gs_image_class_0_interpolate);
 
 /* If we're interpolating, use special logic. */
-private irender_proc(image_render_interpolate);
+static irender_proc(image_render_interpolate);
 irender_proc_t
 gs_image_class_0_interpolate(gx_image_enum * penum)
 {
@@ -185,7 +185,7 @@ gs_image_class_0_interpolate(gx_image_enum * penum)
 
 /* ------ Rendering for interpolated images ------ */
 
-private int
+static int
 image_render_interpolate(gx_image_enum * penum, const byte * buffer,
 			 int data_x, uint iw, int h, gx_device * dev)
 {
