@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: idstack.c 8062 2007-06-19 05:45:58Z leonardo $ */
+/* $Id: idstack.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Implementation of dictionary stacks */
 #include "ghost.h"
 #include "idict.h"
@@ -198,7 +198,7 @@ dstack_find_name_by_index(dict_stack_t * pds, uint nidx)
 
 /* Set the cached values computed from the top entry on the dstack. */
 /* See idstack.h for details. */
-private const ref_packed no_packed_keys[2] =
+static const ref_packed no_packed_keys[2] =
 {packed_key_deleted, packed_key_empty};
 void
 dstack_set_top(dict_stack_t * pds)
