@@ -1,26 +1,21 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: igc.c,v 1.5.2.1.2.1 2003/01/17 00:49:04 giles Exp $ */
+/* $Id: igc.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "gsexit.h"
 #include "gsmdebug.h"
 #include "gsstruct.h"
@@ -37,13 +32,10 @@
 #include "opdef.h"		/* for marking oparray names */
 
 /* Define whether to force all garbage collections to be global. */
-private const bool I_FORCE_GLOBAL_GC = false;
+static const bool I_FORCE_GLOBAL_GC = false;
 
 /* Define whether to bypass the collector entirely. */
-private const bool I_BYPASS_GC = false;
-
-/* Avoid including all of iname.h. */
-extern name_table *the_gs_name_table;
+static const bool I_BYPASS_GC = false;
 
 /* Define an entry on the mark stack. */
 typedef struct {
@@ -72,35 +64,36 @@ struct gc_mark_stack_s {
 #define ms_size_min 50		/* min size for segment in free block */
 
 /* Forward references */
-private void gc_init_mark_stack(P2(gc_mark_stack *, uint));
-private void gc_objects_clear_marks(P1(chunk_t *));
-private void gc_unmark_names(P1(name_table *));
-private int gc_trace(P3(gs_gc_root_t *, gc_state_t *, gc_mark_stack *));
-private int gc_rescan_chunk(P3(chunk_t *, gc_state_t *, gc_mark_stack *));
-private int gc_trace_chunk(P3(chunk_t *, gc_state_t *, gc_mark_stack *));
-private bool gc_trace_finish(P1(gc_state_t *));
-private void gc_clear_reloc(P1(chunk_t *));
-private void gc_objects_set_reloc(P1(chunk_t *));
-private void gc_do_reloc(P3(chunk_t *, gs_ref_memory_t *, gc_state_t *));
-private void gc_objects_compact(P2(chunk_t *, gc_state_t *));
-private void gc_free_empty_chunks(P1(gs_ref_memory_t *));
+
+static void gc_init_mark_stack(gc_mark_stack *, uint);
+static void gc_objects_clear_marks(const gs_memory_t *mem, chunk_t *);
+static void gc_unmark_names(name_table *);
+static int gc_trace(gs_gc_root_t *, gc_state_t *, gc_mark_stack *);
+static int gc_rescan_chunk(chunk_t *, gc_state_t *, gc_mark_stack *);
+static int gc_trace_chunk(const gs_memory_t *mem, chunk_t *, gc_state_t *, gc_mark_stack *);
+static bool gc_trace_finish(gc_state_t *);
+static void gc_clear_reloc(chunk_t *);
+static void gc_objects_set_reloc(chunk_t *);
+static void gc_do_reloc(chunk_t *, gs_ref_memory_t *, gc_state_t *);
+static void gc_objects_compact(chunk_t *, gc_state_t *);
+static void gc_free_empty_chunks(gs_ref_memory_t *);
 
 /* Forward references for pointer types */
-private ptr_proc_unmark(ptr_struct_unmark);
-private ptr_proc_mark(ptr_struct_mark);
-private ptr_proc_unmark(ptr_string_unmark);
-private ptr_proc_mark(ptr_string_mark);
+static ptr_proc_unmark(ptr_struct_unmark);
+static ptr_proc_mark(ptr_struct_mark);
+static ptr_proc_unmark(ptr_string_unmark);
+static ptr_proc_mark(ptr_string_mark);
 /*ptr_proc_unmark(ptr_ref_unmark); *//* in igc.h */
 /*ptr_proc_mark(ptr_ref_mark); *//* in igc.h */
-private ptr_proc_reloc(igc_reloc_struct_ptr, void);
+static ptr_proc_reloc(igc_reloc_struct_ptr, void);
 
 ptr_proc_reloc(igc_reloc_ref_ptr, ref_packed);	/* in igcref.c */
 refs_proc_reloc(igc_reloc_refs);	/* in igcref.c */
 
 /* Define this GC's procedure vector. */
-private const gc_procs_with_refs_t igc_procs = {
+static const gc_procs_with_refs_t igc_procs = {
     igc_reloc_struct_ptr, igc_reloc_string, igc_reloc_const_string,
-    igc_reloc_ref_ptr, igc_reloc_refs
+    igc_reloc_param_string, igc_reloc_ref_ptr, igc_reloc_refs
 };
 
 /* Pointer type descriptors. */
@@ -122,7 +115,7 @@ const gs_ptr_procs_t ptr_ref_procs =
 
 /* Top level of garbage collector. */
 #ifdef DEBUG
-private void
+static void
 end_phase(const char *str)
 {
     if (gs_debug_c('6')) {
@@ -132,7 +125,7 @@ end_phase(const char *str)
     }
 }
 static const char *const depth_dots_string = "..........";
-private const char *
+static const char *
 depth_dots(const ms_entry * sp, const gc_mark_stack * pms)
 {
     int depth = sp - pms->entries - 1;
@@ -142,7 +135,7 @@ depth_dots(const ms_entry * sp, const gc_mark_stack * pms)
 	depth += pss->count - 1;
     return depth_dots_string + (depth >= 10 ? 0 : 10 - depth);
 }
-private void
+static void
 gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
 {
     int i;
@@ -155,6 +148,7 @@ gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
 #else  /* !DEBUG */
 #  define end_phase(str) DO_NOTHING
 #endif /* DEBUG */
+
 void
 gs_gc_reclaim(vm_spaces * pspaces, bool global)
 {
@@ -176,6 +170,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 	ms_entry body[ms_size_default];
     } ms_default;
     gc_mark_stack *mark_stack = &ms_default.stack;
+    const gs_memory_t *cmem;
 
     /* Optionally force global GC for debugging. */
 
@@ -185,6 +180,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     /* Determine which spaces we are tracing and collecting. */
 
     spaces = *pspaces;
+    cmem = space_system->stable_memory;
     space_memories[1] = space_system;
     space_memories[2] = space_global;
     min_collect = max_trace = 2;
@@ -230,8 +226,8 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     state.spaces = spaces;
     state.min_collect = min_collect_vm_space << r_space_shift;
     state.relocating_untraced = false;
-    state.heap = state.loc.memory->parent;
-    state.ntable = the_gs_name_table;
+    state.heap = state.loc.memory->non_gc_memory;
+    state.ntable = state.heap->gs_lib_ctx->gs_name_table;
 
     /* Register the allocators themselves as roots, */
     /* so we mark and relocate the change and save lists properly. */
@@ -262,7 +258,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
     for_collected_spaces(ispace)
 	for_space_chunks(ispace, mem, cp) {
-	gc_objects_clear_marks(cp);
+        gc_objects_clear_marks((const gs_memory_t *)mem, cp);
 	gc_strings_set_marks(cp, false);
     }
 
@@ -336,7 +332,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
 	if (!global)
 	    for_chunks(min_collect - 1, mem, cp)
-		more |= gc_trace_chunk(cp, &state, mark_stack);
+		more |= gc_trace_chunk((const gs_memory_t *)mem, cp, &state, mark_stack);
 
 	/* Handle mark stack overflow. */
 
@@ -376,12 +372,24 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
 	end_phase("finish trace");
     }
+
+    /* Filter save change lists with removing elements,
+       which point to unmarked blocks of refs. */
+    {
+	int i;
+
+	for_collected_spaces(i) {
+	    gs_ref_memory_t *mem = space_memories[i];
+
+	    alloc_save__filter_changes(mem);
+	}
+    }
     /* Clear marks and relocation in spaces that are only being traced. */
     /* We have to clear the marks first, because we want the */
     /* relocation to wind up as o_untraced, not o_unmarked. */
 
     for_chunks(min_collect - 1, mem, cp)
-	gc_objects_clear_marks(cp);
+        gc_objects_clear_marks((const gs_memory_t *)mem, cp);
 
     end_phase("post-clear marks");
 
@@ -513,7 +521,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 		      (ulong) mem, total.allocated, total.used);
 	    gs_memory_status((gs_memory_t *) mem, &total);
 	    mem->gc_allocated = mem->allocated + total.allocated;
-	    mem->inherited = -mem->allocated;
+	    mem->inherited = -(int)mem->allocated;
 	}
 	mem = space_memories[ispace];
 	mem->previous_status = total;
@@ -558,7 +566,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 /* ------ Unmarking phase ------ */
 
 /* Unmark a single struct. */
-private void
+static void
 ptr_struct_unmark(enum_ptr_t *pep, gc_state_t * ignored)
 {
     void *const vptr = (void *)pep->ptr; /* break const */
@@ -568,15 +576,15 @@ ptr_struct_unmark(enum_ptr_t *pep, gc_state_t * ignored)
 }
 
 /* Unmark a single string. */
-private void
+static void
 ptr_string_unmark(enum_ptr_t *pep, gc_state_t * gcst)
 {
     discard(gc_string_mark(pep->ptr, pep->size, false, gcst));
 }
 
 /* Unmark the objects in a chunk. */
-private void
-gc_objects_clear_marks(chunk_t * cp)
+static void
+gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
 {
     if_debug_chunk('6', "[6]unmarking chunk", cp);
     SCAN_CHUNK_OBJECTS(cp)
@@ -592,13 +600,13 @@ gc_objects_clear_marks(chunk_t * cp)
 	      (ulong) size, (ulong) pre);
     o_set_unmarked(pre);
     if (proc != 0)
-	(*proc) (pre + 1, size, pre->o_type);
+        (*proc) (mem, pre + 1, size, pre->o_type);
     END_OBJECTS_SCAN
 }
 
 /* Mark 0- and 1-character names, and those referenced from the */
 /* op_array_nx_table, and unmark all the rest. */
-private void
+static void
 gc_unmark_names(name_table * nt)
 {
     uint i;
@@ -619,7 +627,7 @@ gc_unmark_names(name_table * nt)
 /* ------ Marking phase ------ */
 
 /* Initialize (a segment of) the mark stack. */
-private void
+static void
 gc_init_mark_stack(gc_mark_stack * pms, uint count)
 {
     pms->next = 0;
@@ -631,7 +639,7 @@ gc_init_mark_stack(gc_mark_stack * pms, uint count)
 
 /* Mark starting from all marked objects in the interval of a chunk */
 /* needing rescanning. */
-private int
+static int
 gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     byte *sbot = cp->rescan_bot;
@@ -639,6 +647,7 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
     gs_gc_root_t root;
     void *comp;
     int more = 0;
+    const gs_memory_t *mem = gcst_get_memory_ptr( pstate );
 
     if (sbot > stop)
 	return 0;
@@ -686,7 +695,7 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 	    if (!o_is_untraced(pre))
 		o_set_unmarked(pre);
 	    if (proc != 0)
-		(*proc) (comp, size, pre->o_type);
+	        (*proc) (mem, comp, size, pre->o_type);
 	    more |= gc_trace(&root, pstate, pmstack);
 	}
     }
@@ -697,8 +706,8 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 /* Mark starting from all the objects in a chunk. */
 /* We assume that pstate->min_collect > avm_system, */
 /* so we don't have to trace names. */
-private int
-gc_trace_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
+static int
+gc_trace_chunk(const gs_memory_t *mem, chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     gs_gc_root_t root;
     void *comp;
@@ -741,7 +750,7 @@ gc_trace_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 		root.ptype = ptr_struct_type;
 		comp = pre + 1;
 		if (proc != 0)
-		    (*proc) (comp, size, pre->o_type);
+		    (*proc) (mem, comp, size, pre->o_type);
 		more |= gc_trace(&root, pstate, pmstack);
 	    }
 	}
@@ -754,8 +763,8 @@ gc_trace_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 /* Return -1 if we overflowed the mark stack, */
 /* 0 if we completed successfully without marking any new objects, */
 /* 1 if we completed and marked some new objects. */
-private int gc_extend_stack(P2(gc_mark_stack *, gc_state_t *));
-private int
+static int gc_extend_stack(gc_mark_stack *, gc_state_t *);
+static int
 gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     int min_trace = pstate->min_collect;
@@ -827,7 +836,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
 	    mproc = ptr[-1].o_type->enum_ptrs;
 	    if (mproc == gs_no_struct_enum_ptrs ||
 		(ptp = (*mproc)
-		 (ptr, pre_obj_contents_size(ptr - 1),
+		 (gcst_get_memory_ptr(pstate), ptr, pre_obj_contents_size(ptr - 1),
 		  sp->index, &nep, ptr[-1].o_type, pstate)) == 0
 		) {
 		if_debug0('7', " - done\n");
@@ -1007,7 +1016,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
 }
 /* Link to, attempting to allocate if necessary, */
 /* another chunk of mark stack. */
-private int
+static int
 gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 {
     if (pms->next == 0) {	/* Try to allocate another segment. */
@@ -1032,7 +1041,7 @@ gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 		/* storage.  This can't happen. */
 		lprintf1("mark stack overflowed while outside collectible space at 0x%lx!\n",
 			 (ulong) cptr);
-		gs_abort();
+		gs_abort(pstate->heap);
 	    }
 	    if (cptr < cp->rescan_bot)
 		cp->rescan_bot = cptr, new = -1;
@@ -1048,7 +1057,7 @@ gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 }
 
 /* Mark a struct.  Return true if new mark. */
-private bool
+static bool
 ptr_struct_mark(enum_ptr_t *pep, gc_state_t * ignored)
 {
     obj_header_t *ptr = (obj_header_t *)pep->ptr;
@@ -1063,14 +1072,14 @@ ptr_struct_mark(enum_ptr_t *pep, gc_state_t * ignored)
 }
 
 /* Mark a string.  Return true if new mark. */
-private bool
+static bool
 ptr_string_mark(enum_ptr_t *pep, gc_state_t * gcst)
 {
     return gc_string_mark(pep->ptr, pep->size, true, gcst);
 }
 
 /* Finish tracing by marking names. */
-private bool
+static bool
 gc_trace_finish(gc_state_t * pstate)
 {
     name_table *nt = pstate->ntable;
@@ -1101,7 +1110,7 @@ gc_trace_finish(gc_state_t * pstate)
 /* ------ Relocation planning phase ------ */
 
 /* Initialize the relocation information in the chunk header. */
-private void
+static void
 gc_init_reloc(chunk_t * cp)
 {
     chunk_head_t *chead = cp->chead;
@@ -1114,7 +1123,7 @@ gc_init_reloc(chunk_t * cp)
 }
 
 /* Set marks and clear relocation for chunks that won't be compacted. */
-private void
+static void
 gc_clear_reloc(chunk_t * cp)
 {
     byte *pfree = (byte *) & cp->chead->free;
@@ -1136,7 +1145,7 @@ gc_clear_reloc(chunk_t * cp)
 
 /* Set the relocation for the objects in a chunk. */
 /* This will never be called for a chunk with any o_untraced objects. */
-private void
+static void
 gc_objects_set_reloc(chunk_t * cp)
 {
     uint reloc = 0;
@@ -1182,7 +1191,7 @@ gc_objects_set_reloc(chunk_t * cp)
 /* ------ Relocation phase ------ */
 
 /* Relocate the pointers in all the objects in a chunk. */
-private void
+static void
 gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
 {
     chunk_head_t *chead = cp->chead;
@@ -1190,6 +1199,9 @@ gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
     if_debug_chunk('6', "[6]relocating in chunk", cp);
     SCAN_CHUNK_OBJECTS(cp)
 	DO_ALL
+#ifdef DEBUG
+	pstate->container = cp;
+#endif
     /* We need to relocate the pointers in an object iff */
     /* it is o_untraced, or it is a useful object. */
     /* An object is free iff its back pointer points to */
@@ -1207,6 +1219,9 @@ gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
 	    if (proc != 0)
 		(*proc) (pre + 1, size, pre->o_type, pstate);
 	}
+#ifdef DEBUG
+	pstate->container = 0;
+#endif
     END_OBJECTS_SCAN
 }
 
@@ -1223,7 +1238,7 @@ print_reloc_proc(const void *obj, const char *cname, const void *robj)
 
 /* Relocate a pointer to an (aligned) object. */
 /* See gsmemory.h for why the argument is const and the result is not. */
-private void /*obj_header_t */ *
+static void /*obj_header_t */ *
 igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 {
     const obj_header_t *const optr = (const obj_header_t *)obj;
@@ -1242,10 +1257,16 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 	else {
 #ifdef DEBUG
 	    /* Do some sanity checking. */
-	    if (back > gcst->space_local->chunk_size >> obj_back_shift) {
-		lprintf2("Invalid back pointer %u at 0x%lx!\n",
-			 back, (ulong) obj);
-		gs_abort();
+	    chunk_t *cp = gcst->container;
+
+	    if (cp != 0 && cp->cbase <= (byte *)obj && (byte *)obj <cp->ctop) {
+		if (back > (cp->ctop - cp->cbase) >> obj_back_shift) {
+		    lprintf2("Invalid back pointer %u at 0x%lx!\n",
+			     back, (ulong) obj);
+		    gs_abort(NULL);
+		}
+	    } else {
+		/* Pointed to unknown chunk. Can't check it, sorry. */
 	    }
 #endif
 	    {
@@ -1275,11 +1296,12 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 
 /* Compact the objects in a chunk. */
 /* This will never be called for a chunk with any o_untraced objects. */
-private void
+static void
 gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 {
     chunk_head_t *chead = cp->chead;
     obj_header_t *dpre = (obj_header_t *) chead->dest;
+    const gs_memory_t *cmem = gcst->spaces.memories.named.system->stable_memory;
 
     SCAN_CHUNK_OBJECTS(cp)
 	DO_ALL
@@ -1298,7 +1320,7 @@ gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 		memmove(dpre, pre,
 			sizeof(obj_header_t) + size);
 	} else
-	    (*procs->compact) (pre, dpre, size);
+	    (*procs->compact) (cmem, pre, dpre, size);
 	dpre = (obj_header_t *)
 	    ((byte *) dpre + obj_size_round(size));
     }
@@ -1314,7 +1336,7 @@ gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 /* ------ Cleanup ------ */
 
 /* Free empty chunks. */
-private void
+static void
 gc_free_empty_chunks(gs_ref_memory_t * mem)
 {
     chunk_t *cp;
@@ -1334,3 +1356,11 @@ gc_free_empty_chunks(gs_ref_memory_t * mem)
 	}
     }
 }
+
+
+const gs_memory_t * gcst_get_memory_ptr(gc_state_t *gcst)
+{
+    vm_spaces spaces = gcst->spaces;
+    const gs_memory_t *cmem = space_system->stable_memory;	
+    return cmem;
+}
