@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ireclaim.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: ireclaim.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Interpreter's interface to garbage collector */
 #include "ghost.h"
 #include "ierrors.h"
@@ -31,11 +31,11 @@
 extern void ialloc_gc_prepare(gs_ref_memory_t *);
 
 /* Forward references */
-private void gs_vmreclaim(gs_dual_memory_t *, bool);
+static void gs_vmreclaim(gs_dual_memory_t *, bool);
 
 /* Initialize the GC hook in the allocator. */
-private int ireclaim(gs_dual_memory_t *, int);
-private int
+static int ireclaim(gs_dual_memory_t *, int);
+static int
 ireclaim_init(i_ctx_t *i_ctx_p)
 {
     gs_imemory.reclaim = ireclaim;
@@ -44,7 +44,7 @@ ireclaim_init(i_ctx_t *i_ctx_p)
 
 /* GC hook called when the allocator signals a GC is needed (space = -1), */
 /* or for vmreclaim (space = the space to collect). */
-private int
+static int
 ireclaim(gs_dual_memory_t * dmem, int space)
 {
     bool global;
@@ -95,7 +95,7 @@ ireclaim(gs_dual_memory_t * dmem, int space)
 }
 
 /* Interpreter entry to garbage collector. */
-private void
+static void
 gs_vmreclaim(gs_dual_memory_t *dmem, bool global)
 {
     /* HACK: we know the gs_dual_memory_t is embedded in a context state. */
