@@ -1,22 +1,17 @@
-/* Copyright (C) 1993, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: sfilter1.c,v 1.6.4.1.2.1 2003/01/17 00:49:05 giles Exp $ */
+/* $Id: sfilter1.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Filters included in Level 1 systems: NullEncode/Decode, PFBDecode, */
 /*   SubFileDecode. */
 #include "stdio_.h"		/* includes std.h */
@@ -29,7 +24,7 @@
 private_st_PFBD_state();
 
 /* Initialize the state */
-private int
+static int
 s_PFBD_init(stream_state * st)
 {
     stream_PFBD_state *const ss = (stream_PFBD_state *) st;
@@ -39,7 +34,7 @@ s_PFBD_init(stream_state * st)
 }
 
 /* Process a buffer */
-private int
+static int
 s_PFBD_process(stream_state * st, stream_cursor_read * pr,
 	       stream_cursor_write * pw, bool last)
 {
@@ -150,7 +145,7 @@ const stream_template s_PFBD_template = {
 private_st_SFD_state();
 
 /* Set default parameter values. */
-private void
+static void
 s_SFD_set_defaults(stream_state * st)
 {
     stream_SFD_state *const ss = (stream_SFD_state *) st;
@@ -162,7 +157,7 @@ s_SFD_set_defaults(stream_state * st)
 }
 
 /* Initialize the stream */
-private int
+static int
 s_SFD_init(stream_state * st)
 {
     stream_SFD_state *const ss = (stream_SFD_state *) st;
@@ -175,7 +170,7 @@ s_SFD_init(stream_state * st)
 }
 
 /* Refill the buffer */
-private int
+static int
 s_SFD_process(stream_state * st, stream_cursor_read * pr,
 	      stream_cursor_write * pw, bool last)
 {
