@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: sjbig2_luratech.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: sjbig2_luratech.c 8250 2007-09-25 13:31:24Z giles $ */
 /* jbig2decode filter implementation -- hooks in luratech JBIG2 */
 
 #include "memory_.h"
@@ -59,7 +59,7 @@ typedef struct s_jbig2decode_global_data_s {
 } s_jbig2decode_global_data;
 
 /* create a global data struct and copy data into it */
-public int
+int
 s_jbig2decode_make_global_data(byte *data, uint size, void **result)
 {
     s_jbig2decode_global_data *global = NULL;
@@ -80,7 +80,7 @@ s_jbig2decode_make_global_data(byte *data, uint size, void **result)
 }
 
 /* free a global data struct and its data */
-public void
+void
 s_jbig2decode_free_global_data(void *data)
 {
     s_jbig2decode_global_data *global = (s_jbig2decode_global_data*)data;
@@ -93,7 +93,7 @@ s_jbig2decode_free_global_data(void *data)
 }
 
 /* store a global ctx pointer in our state structure */
-public int
+int
 s_jbig2decode_set_global_data(stream_state *ss, void *data)
 {
     stream_jbig2decode_state *state = (stream_jbig2decode_state*)ss;
@@ -117,7 +117,7 @@ s_jbig2decode_set_global_data(stream_state *ss, void *data)
 /* invert the bits in a buffer */
 /* jbig2 and postscript have different senses of what pixel
    value is black, so we must invert the image */
-private void
+static void
 s_jbig2_invert_buffer(unsigned char *buf, int length)
 {
     int i;
@@ -129,7 +129,7 @@ s_jbig2_invert_buffer(unsigned char *buf, int length)
 /** callbacks passed to the luratech library */
 
 /* memory allocator */
-private void * JB2_Callback
+static void * JB2_Callback
 s_jbig2_alloc(unsigned long size, void *userdata)
 {
     void *result = malloc(size);
@@ -137,7 +137,7 @@ s_jbig2_alloc(unsigned long size, void *userdata)
 }
 
 /* memory release */
-private JB2_Error JB2_Callback
+static JB2_Error JB2_Callback
 s_jbig2_free(void *ptr, void *userdata)
 {
     free(ptr);
@@ -145,7 +145,7 @@ s_jbig2_free(void *ptr, void *userdata)
 }
 
 /* error callback for jbig2 codec */
-private void JB2_Callback
+static void JB2_Callback
 s_jbig2_message(const char *message, JB2_Message_Level level, void *userdata)
 {
     const char *type;
@@ -174,7 +174,7 @@ s_jbig2_message(const char *message, JB2_Message_Level level, void *userdata)
 }
 
 /* compressed read callback for jbig2 codec */
-private JB2_Size_T JB2_Callback
+static JB2_Size_T JB2_Callback
 s_jbig2_read(unsigned char *buffer, 
 	JB2_Size_T offset, JB2_Size_T size, void *userdata)
 {
@@ -200,7 +200,7 @@ s_jbig2_read(unsigned char *buffer,
 }
 
 /* uncompressed write callback for jbig2 codec */
-private JB2_Error JB2_Callback
+static JB2_Error JB2_Callback
 s_jbig2_write(unsigned char *buffer,
 		unsigned long row, unsigned long width,
 		unsigned long bbp, void *userdata)
@@ -221,7 +221,7 @@ s_jbig2_write(unsigned char *buffer,
 }
 
 
-private int
+static int
 s_jbig2decode_inbuf(stream_jbig2decode_state *state, stream_cursor_read * pr)
 {
     long in_size = pr->limit - pr->ptr;
@@ -262,7 +262,7 @@ s_jbig2decode_inbuf(stream_jbig2decode_state *state, stream_cursor_read * pr)
 }
 
 /* initialize the steam. */
-private int
+static int
 s_jbig2decode_init(stream_state * ss)
 {
     stream_jbig2decode_state *const state = (stream_jbig2decode_state *) ss;
@@ -285,7 +285,7 @@ s_jbig2decode_init(stream_state * ss)
 /* process a section of the input and return any decoded data.
    see strimpl.h for return codes.
  */
-private int
+static int
 s_jbig2decode_process(stream_state * ss, stream_cursor_read * pr,
 		  stream_cursor_write * pw, bool last)
 {
@@ -365,7 +365,7 @@ s_jbig2decode_process(stream_state * ss, stream_cursor_read * pr,
 }
 
 /* stream release. free all our decoder state. */
-private void
+static void
 s_jbig2decode_release(stream_state *ss)
 {
     stream_jbig2decode_state *const state = (stream_jbig2decode_state *) ss;
@@ -398,7 +398,7 @@ const stream_template s_jbig2decode_template = {
 private_st_jbig2encode_state();
 
 /* helper - start up the compression context */
-private int
+static int
 s_jbig2encode_start(stream_jbig2encode_state *state)
 {
     JB2_Error err;
@@ -431,7 +431,7 @@ s_jbig2encode_start(stream_jbig2encode_state *state)
 }
 
 /* callback for compressed data output */
-private JB2_Size_T JB2_Callback
+static JB2_Size_T JB2_Callback
 s_jbig2encode_write(const unsigned char *buffer,
 		JB2_Size_T pos, JB2_Size_T size, void *userdata)
 {
@@ -468,7 +468,7 @@ s_jbig2encode_write(const unsigned char *buffer,
 
 
 /* initialize the steam. */
-private int
+static int
 s_jbig2encode_init(stream_state * ss)
 {
     stream_jbig2encode_state *state = (stream_jbig2encode_state *)ss;
@@ -497,7 +497,7 @@ s_jbig2encode_init(stream_state * ss)
 /* process a section of the input and return any encoded data.
    see strimpl.h for return codes.
  */
-private int
+static int
 s_jbig2encode_process(stream_state * ss, stream_cursor_read * pr,
                   stream_cursor_write * pw, bool last)
 {
@@ -581,7 +581,7 @@ s_jbig2encode_process(stream_state * ss, stream_cursor_read * pr,
 
 /* stream release. free all our decoder state.
  */
-private void
+static void
 s_jbig2encode_release(stream_state *ss)
 {
     stream_jbig2encode_state *state = (stream_jbig2encode_state *)ss;
