@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: spprint.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: spprint.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Print values in ASCII form on a stream */
 #include "math_.h"		/* for fabs */
 #include "stdio_.h"		/* for stream.h */
@@ -44,7 +44,7 @@ stream_puts(stream * s, const char *str)
 
 /* Print a format string up to the first variable substitution. */
 /* Return a pointer to the %, or to the terminating 0 if no % found. */
-private const char *
+static const char *
 pprintf_scan(stream * s, const char *format)
 {
     const char *fp = format;
@@ -61,7 +61,7 @@ pprintf_scan(stream * s, const char *format)
 }
 
 /* Print a short string on a stream. */
-private void
+static void
 pputs_short(stream *s, const char *str)
 {
     const char *p = str;
