@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfontenum.c 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: zfontenum.c 8250 2007-09-25 13:31:24Z giles $ */
 
 /* this is the ps interpreter interface to the native font
    enumeration code. it calls the platform-specific routines
@@ -39,7 +39,7 @@ typedef struct fontenum_s {
 } fontenum_t;
 
 /* .getnativefonts [ [<name> <path>] ... ] */
-private int
+static int
 z_fontenum(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -125,8 +125,8 @@ z_fontenum(i_ctx_t *i_ctx_p)
 }
 
 
-/* match the above routines to their postscript filter names
-   this is how our 'private' routines get called externally */
+/* Match the above routines to their postscript filter names.
+   This is how our static routines get called externally. */
 const op_def zfontenum_op_defs[] = {
     {"0.getnativefonts", z_fontenum},
     op_def_end(0)
